function [TraceGroup_FootprintVector, TraceGroup_SystemFootprint, FootprintArray] = f_Footprint_07272018(ECOTRAN, TargetGroup_Footprint, TraceNonProduction, use_MonteCarlo)
% Calculate Production Required (FOOTPRINT)
% NOTE: uses ECOTRAN
% NOTE: excludes phytoplankton, micrograzers, and microbes from denominator in system footprint calc
% takes:
%       ECOTRAN
%               GroupType
%               EnergyBudget_matrix
%               GroupTypeDef_ (several...)
%               EwE_PB
%               EwE_Biomass
%       TargetGroup_Footprint       target group for Footprint analysis (footprint TargetGroup is a consumer)
%       TraceNonProduction          'y' or 'n'; y = account for non-production consumption to include "other" mortality & metabolic costs of each trophic link
%       use_MonteCarlo              'y' or 'n'
% returns:
%       TraceGroup_FootprintVector  fraction of each producer group flowing to consumer = TargetGroup_Footprint; 
%                                       2 horizontal vectors: 
%                                           row 1 = mean across MonteCarlo models, 
%                                           row 2 = std across MonteCarlo models,
%                                           clms  = producer groups
%                                       (NOTE: use for web plotting footprint box colors)
%       TraceGroup_SystemFootprint  SYSTEM-LEVEL footprint = ratio of total TraceGroup footprint on all PRODUCERS to total production of all PRODUCER groups
%                                       2 horizontal vectors: 
%                                           row 1 = mean across MonteCarlo models,
%                                           row 2 = std across MonteCarlo models,
%                                           clms  = consumer groups
%       FootprintArray              fraction of each producer group flowing to each consumer 
%                                       (3D stack of matrices; rows = consumers X clms = producer X layers = MonteCarlo models)
% calls:
%       f_ProductionTrace_07272018
% revision date: 7-27-2018


% *************************************************************************
% STEP 1: unpack ECOTRAN.GroupType and get size info-----------------------
ECOTRAN_GroupType                        = ECOTRAN.GroupType;
[ECOTRAN_rows, ECOTRAN_clms, num_trials] = size(ECOTRAN.EnergyBudget_matrix); % number of separate fisheries





% *************************************************************************
% STEP 2: Production Required (FOOTPRINT)----------------------------------
% step 2a: initialize -----------------------------------------------------
FootprintArray      = zeros(ECOTRAN_clms, ECOTRAN_clms, num_trials);
mean_Footprint      = zeros(ECOTRAN_clms, ECOTRAN_clms, 1);
std_Footprint       = zeros(ECOTRAN_clms, ECOTRAN_clms, 1);

% step 2b: calculate footprint on each consumer (rows) on each producer (clms)
for producer_loop = 1:ECOTRAN_clms
    current_ProducerGrp                                         = producer_loop;
    [ProductionTrace_matrix, Footprint_vector]                  = f_ProductionTrace_07272018(ECOTRAN, current_ProducerGrp, TraceNonProduction);
        % Footprint_vector = fraction of PRODUCER = current_ProducerGrp production flowing to each consumer group (vertical vectors: 1 clm for each MonteCarlo model)
        %       NOTE: this vector tells the footprint of each consumer group (rows) upon PRODUCER = current_ProducerGrp
        % ProductionTrace_matrix = not really usable for web-plotting
    
    Footprint_vector(producer_loop, :)                          = 0; % set footprint of TraceGroup self in Footprint_vector to ZERO

    mean_Footprint(1:ECOTRAN_clms, producer_loop)               = mean(Footprint_vector, 2);	% average footprint upon current_ProducerGrp across all MonteCarlo models; (vertical vectors; 1 clm for each producr)
    std_Footprint(1:ECOTRAN_clms, producer_loop)                = std(Footprint_vector, [], 2);	% (vertical vectors; 1 clm for each producr)
    
    Footprint_vector                                            = reshape(Footprint_vector, ECOTRAN_clms, 1, num_trials); % reshape Footprint_vector to build up 3D FootprintArray (vertical vectors; 1 clm stacked in layers for each MonteCarlo model)
    FootprintArray(1:ECOTRAN_clms, producer_loop, 1:num_trials) = Footprint_vector;             % (stack of matrices; 1 clm for each producer, 1 layer for each MonteCarlo model)
end

% step 2c: package footprint of TargetGroup_Footprint for export (mean across MonteCarlo models)
temp_mean_FootprintVector                               = FootprintArray(TargetGroup_Footprint, :, :); % pick TargetGroup_Footprint as a CONSUMER row in FootprintArray
temp_mean_FootprintVector(1, TargetGroup_Footprint, :)  = 1; % set footprint on self = 1
result_mean_FootprintVector                             = mean(temp_mean_FootprintVector, 3); % (horizontal vector)
result_std_FootprintVector                              = std(temp_mean_FootprintVector, [], 3); % (horizontal vector)
% package footprint of TargetGroup_Footprint for export
TraceGroup_FootprintVector(1, 1:ECOTRAN_clms)           = result_mean_FootprintVector;
TraceGroup_FootprintVector(2, 1:ECOTRAN_clms)           = result_std_FootprintVector;





% *************************************************************************
% STEP 3: calculate SYSTEM-LEVEL footprint---------------------------------
% step 3a: get addresses of nutrients, detritus, BA & EM
looky_NUTRIENTS                         = find(floor(ECOTRAN_GroupType) == ECOTRAN.GroupTypeDef_ANYNitroNutr);
looky_DETRITUS                          = find(floor(ECOTRAN_GroupType) == ECOTRAN.GroupTypeDef_ANYDetritus);
looky_CONSUMERS                         = find(floor(ECOTRAN_GroupType) == ECOTRAN.GroupTypeDef_ANYConsumer | floor(ECOTRAN_GroupType) == ECOTRAN.GroupTypeDef_fishery); % find consumer groups
looky_BA                                = find(ECOTRAN_GroupType        == ECOTRAN.GroupTypeDef_BA);
looky_EM                                = find(ECOTRAN_GroupType        == ECOTRAN.GroupTypeDef_EM);
looky_Micrograzers                      = find(ECOTRAN_GroupType        == 3.15 | ECOTRAN_GroupType == 3.25); % find pelagic and benthic micrograzer groups
ECOTRAN_GroupType([looky_BA; looky_EM]) = [];

% step 3b: calculate production rates for each group within each MonteCarlo model
ECOTRAN_production                      = ECOTRAN.EwE_PB .* ECOTRAN.EwE_Biomass; % production vector; (array of vertical vectors: groups X trials)

% step 3c: calculate PRODUCER production ----------------------------------
production_PRODUCER                   	= ECOTRAN_production;

total_production_PRODUCER               = production_PRODUCER;
total_production_PRODUCER([looky_Micrograzers'], :) = 0; % zero-out micrograzers
total_production_PRODUCER              	= total_production_PRODUCER([looky_CONSUMERS'], :); % only consider consumer production
total_production_PRODUCER              	= sum(total_production_PRODUCER); % total production of all PRODUCER groups (scaler or horizontal vector of length trial_num)

production_PRODUCER([looky_NUTRIENTS' looky_DETRITUS'], :) = []; % remove non-producer group rows (nutrients, detritus, import)


Footprint_array_PRODUCER                                   = FootprintArray;
Footprint_array_PRODUCER(:, [looky_NUTRIENTS' looky_DETRITUS'], :) = [];
if strcmp(use_MonteCarlo, 'y') 
    Footprint_vector_PRODUCER = squeeze(Footprint_array_PRODUCER(TargetGroup_Footprint, :, :))'; % (2D matrix: trial_num X PRODUCER)
else
    Footprint_vector_PRODUCER = Footprint_array_PRODUCER(TargetGroup_Footprint, :); % (horizontal vector: 1 X (ECOTRAN_rows minus nutrients & detritus grps))
end

TraceGroup_FootprintProduction                              = Footprint_vector_PRODUCER .* production_PRODUCER'; % footprint of TraceGroup upon each producer in terms of production; Footprint_vector_PRODUCER * (PRODUCER production) = (net or gross production of each PRODUCER going to TraceGroup CONSUMER) (array of horizontal vectors, trial_num X PRODUCER)
% TraceGroup_FootprintProduction       = Footprint_vector_PRODUCER .* production_PRODUCER; % note change of transpose in this error fix
TraceGroup_FootprintProduction_total                        = sum(TraceGroup_FootprintProduction'); % total TraceGroup footprint on all PRODUCERS (scaler or horizontal vector of length trial_num)
temp_SystemFootprint                                        = TraceGroup_FootprintProduction_total ./ total_production_PRODUCER; % TraceGroup footprint fraction across all producer groups (scaler or horizontal vector of length trial_num)

% step 3d: mean and std of temp_SystemFootprint across MonteCarlo models --
TraceGroup_SystemFootprint(1, 1) = nanmean(temp_SystemFootprint);
TraceGroup_SystemFootprint(2, 1) = nanstd(temp_SystemFootprint);


% end m-file***************************************************************