function [ProductionTrace_matrix, Footprint_vector] = f_ProductionTrace_07272018(ECOTRAN, TraceGroup, TraceNonProduction)
% trace producer group (p) production through static model food web
% code blocks nutrient recycling in ALL cases
% code automatically tests if TraceGroup is a detritus group and blocks trophic recycling if TraceGroup is anything but detritus
% by Jim Ruzicka
% takes:
%   ECOTRAN.EnergyBudget_matrix     (stack of matrix)
%   ECOTRAN.TransferEfficiency      (array of vertical vectors)
%   ECOTRAN.Metabolism_fraction     (array of horizontal vectors)
%   ECOTRAN.Feces_fraction          (array of horizontal vectors)
%   ECOTRAN.OtherMortality_fraction (array of horizontal vectors)
%   ECOTRAN.GroupType               (vertical vector)
%	TraceGroup                       (row & clm address within EnergyBudget_matrix of agg group to be traced)
%   TraceNonProduction               ('y' - NonProduction costs in trace; 'n' - do NOT include NonProduction costs in trace)
%        PPR_technique               ('normal'    - include OtherMortality & Metabolic costs in trace)
%                                    ('Essington' - include Metabolic costs in trace, but do NOT include OtherMortality costs in trace as per Essington 2007)
% returns:
%	ProductionTrace_matrix           (fraction of energy within each linkge ultimately originating from TraceGroup (p)) (QQQ needs to be divided by EnergyBudget_matrix)        (stack of matrix)
%	Footprint_vector	             (Footprint = fraction of TraceGroup (p) production contributing to the production of each consumer (c))   (array of vertical vectors: 1 clm for each MonteCarlo model)
% calls:
%	none
% revision date: 7-27-2018

% NOTE: changes from f_ProductionTrace_01252014 are only in adding notes for clarification



% *************************************************************************
% STEP 1: set operating parameters-----------------------------------------
iteration_tolerance = 10^(-10); % iterate until diet_trace stabilizes
iteration_max       = 1000; % maximum number of iterations
PPR_technique       = 'normal'; % 'normal', 'Essington'
if isempty(TraceGroup); error('no such trophic group code number'); end;





% *************************************************************************
% STEP 2: get ECOTRAN info-------------------------------------------------
EnergyBudget_matrix  = ECOTRAN.EnergyBudget_matrix;
TransferEfficiency   = ECOTRAN.TransferEfficiency;
E2E_Metabolism       = ECOTRAN.BioenergeticBudget(3, :, :); % as fraction of consumption; (horizontal vectors in MonteCarlo layers)
E2E_Feces            = ECOTRAN.BioenergeticBudget(2, :, :); % as fraction of consumption; (horizontal vectors in MonteCarlo layers)
E2E_OtherMortality   = ECOTRAN.BioenergeticBudget_ProductionDetail(3, :, :); % as fraction of consumption; (horizontal vectors in MonteCarlo layers)
GroupType            = ECOTRAN.GroupType;





% *************************************************************************
% STEP 3: prepare ProductionMatrix and initialize result matrices----------
% step 3a: find BA, EM, detritus, and nutrients by row address in EnergyBudget_matrix (if they exist)
% NOTE: BA & EM don't exist in EnergyBudget_matrix, they are in the BioenergeticBudget
looky_NUTRIENTS = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYNitroNutr);
looky_DETRITUS  = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYDetritus);
looky_BA        = find(GroupType        == ECOTRAN.GroupTypeDef_BA);
looky_EM        = find(GroupType        == ECOTRAN.GroupTypeDef_EM);

% step 3b: initialize result matrices
[ProductionMatrix_rows, ProductionMatrix_clms, num_trials] = size(EnergyBudget_matrix);
Footprint_vector                                           = zeros(ProductionMatrix_rows, num_trials); % (array of vertical vectors)
ProductionTrace_matrix                                     = zeros(ProductionMatrix_rows, ProductionMatrix_clms, num_trials); % (stack of matrices(

% step 3d: zero-out nutrients & detritus as consumer rows (eliminate nutrient & detritus recycling from analysis)
EnergyBudget_matrix([looky_NUTRIENTS; looky_DETRITUS], :, :) = 0;
% alternative option; zero-out nutrients & detritus as producer columns (eliminate nutrient & detritus recycling from analysis)
% if ~ismember(TraceGroup, looky_DETRITUS)
%     EnergyBudget_matrix(:, looky_DETRITUS, :) = 0; %  block detritus recycling (unless producer TraceGroup is a detritus)
% end
% 
% if ~ismember(TraceGroup, looky_NUTRIENTS)
%     EnergyBudget_matrix(:, looky_NUTRIENTS, :) = 0; %  block nutrient recycling (unless producer TraceGroup is a nutrient)
% end





% *************************************************************************
% STEP 4: run through each Monte Carlo model-------------------------------
for trial_loop = 1:num_trials
    
    % step 4a: get current MonteCarlo model -------------------------------
    current_ProductionMatrix   = EnergyBudget_matrix(:, :, trial_loop); % (matrix)
    current_Metabolism         = E2E_Metabolism(1, :, trial_loop);      % (horizontal vector)
    current_OtherMortality     = E2E_OtherMortality(1, :, trial_loop);  % (horizontal vector)

    % step 4b: account for non-production consumption ---------------------
    %          (include other mortality & metabolic costs of each trophic link; do not include feces because feces is not a production item)
    if strcmp(TraceNonProduction, 'y')
        
        current_TransferEfficiency = TransferEfficiency(trial_loop, :);	% (horizontal vector)
        
        if strcmp(PPR_technique, 'Essington')
            NonProductionScaler = (current_TransferEfficiency + current_Metabolism) ./ current_TransferEfficiency; % Essington excludes Other Mortality
        else
            
            NonProduction                 = current_Metabolism + current_OtherMortality;
            NonProduction(looky_DETRITUS) = 0; % do not trace non-production consumption for detritus groups
            % QQQ What was reason that at this point the sum of current_ProductionMatrix clms not same as current_TransferEfficiency for detritus groups?
            NonProductionScaler           = (current_TransferEfficiency + NonProduction) ./ current_TransferEfficiency;
            
            looky_NOnonProd               = find(NonProductionScaler < 1);
            looky_InfProd                 = find(isinf(NonProductionScaler));
            looky_NaNProd                 = find(isnan(NonProductionScaler));
            
            NonProductionScaler([looky_NOnonProd looky_InfProd looky_NaNProd]) = 1; % don't let NonProductionScaler take production away from system, and correct for div/0 errors
        end
        
        for rows_loop = 1:ProductionMatrix_rows
            current_ProductionMatrix(rows_loop, :) = current_ProductionMatrix(rows_loop, :) .* NonProductionScaler; % redistribute metabolism & other mortality costs among predator rows proportionately
        end
        
        current_ProductionMatrix(isnan(current_ProductionMatrix)) = 0; % filter out any NaNs (caused for groups where TE = 0); line probably not necessary
    
    end % (strcmp(TraceNonProduction, 'y'))
    
    
    
    
    
    % step 4c: get OriginVector using the TraceGroup ----------------------
    OriginVector = current_ProductionMatrix(:, TraceGroup); % trace group-as-prey; (vertical vector)

    % step 4d: initialize -------------------------------------------------
    TraceVector             = OriginVector'; % prepare TraceVector (horizontal vector)
    TraceVector(TraceGroup) = 1; % TraceGroup contribution to production of TraceGroup = 100%
    production_trace        = zeros(ProductionMatrix_rows, ProductionMatrix_clms); % prepare production_trace (matrix)
    trace_fraction_previous = ones(ProductionMatrix_rows, 1); % initialize trace_fraction_previous (vertical vector)
    check_iteration         = 1; % initialize check_iteration
    iteration_count         = 0; % initialize iteration_count

    % step 4e: multiply predation on TraceGroup through Production Matrix, iteratively
    while (check_iteration > iteration_tolerance) && (iteration_count < iteration_max)

        for consumer_loop = 1:ProductionMatrix_rows
            production_trace(consumer_loop, :) = TraceVector .* current_ProductionMatrix(consumer_loop, :); % multiply TraceVector through the diet matrix
        end;

        trace_fraction_current  = sum(production_trace, 2); % fraction of trace group production to the production of each consumer (vertical vector)

        % make new TraceVector
        TraceVector             = trace_fraction_current';
        TraceVector(TraceGroup) = 1;
    
        check_iteration         = max(abs(trace_fraction_previous - trace_fraction_current));
        iteration_count         = iteration_count + 1;
        trace_fraction_previous = trace_fraction_current;

    end

    % step 4f: build-up result arrays -------------------------------------
    trace_fraction_current(TraceGroup) = 1; % set origin group to 1
    Footprint_vector(:, trial_loop)    = trace_fraction_current; % build up array of footprint vertical vectors
    ProductionTrace_matrix(1:ProductionMatrix_rows, 1:ProductionMatrix_clms, trial_loop) = production_trace;
    
end % (trial_loop)





% end m-file***************************************************************