function p_WebPlotter_01032017(EwEResult, DietTrace_upward, TraceFraction_upward, DietTrace_downward, FootprintVector, GRPlabels, x_position, TL_position, grid_switch)
% plot a web plot
% uses the 1 EwE "type" model; does not use MonteCarlo models
% takes:
%   EwEResult
%       EwEResult.Biomass
%       EwEResult.Consumption       (end row is import consumption)
%       EwEResult.Production
%       EwEResult.Landings
%       EwEResult.Discards
%       EwEResult.TL_fishery
%       EwEResult.EwE_code
%       EwEResult.GroupType
%       EwEResult.GroupTypeDefs
%   REACH metrics:
%       DietTrace_upward        fraction of energy within each linkage ultimately originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting reach arrow colors)
%       TraceFraction_upward        fraction of each CONSUMER production originating from PRODUCER = TraceGroup; mean across all MonteCarlo models
%                                       (horizontal vector: 1 X clms of consumers)
%                                       (NOTE: use for web plotting reach box colors)
%   FOOTPRINT metrics:
%       DietTrace_downward      fractional contribution of each linkage to production of CONSUMER = TraceGroup
%                                       (2D matrix: rows of producers X clms of consumers)
%                                       (NOTE: use for web plotting footprint arrow colors)
%       FootprintVector             (fraction of each PRODUCER group's production flowing to TraceGroup = CONSUMER) (horizontal vector: 1 X producers)
%                                        (NOTE: use for web plotting footprint box colors)
%                                        (NOTE: derives from row 1 of TraceGroup_FootprintVector)
%   GRPlabels                       (labels for boxes)
%   x_position                      (x-position of each box)
%   TL_position                     (y-position of each box)
%   grid_switch   ('on' to turn on gridlines for placing boxes)
% returns:
%   1 horrendogram
% calls:
%   none
% revision date: 8-20-2016
%       changes from last revision: just cleaning up aesthetics of code


% *************************************************************************
% STEP 1: set color intensity; line width; minimum flow size to plot-------
BoxIntensity  = 0.4; % intensity of box trace color (smaller than 1 intensifies)
LineIntensity = 0.2; % intensity of flow lines (smaller than 1 intensifies)

minline       = 1;
maxline       = 15;
% minspot       = 7; % QQQ
% maxspot       = 20; % QQQ

minspot       = 10; % QQQ
maxspot       = 22; % QQQ

minflow       = 0.0000005; % log(0.01 + 1); % minimum flow size to plot (0.025)
grayness      = 0.9; % 1 is white, 0 is black

xmin          = 0;
xmax          = max(x_position);
ymin          = 0.5; % QQQ
% ymin          = 0.75;
ymax          = 5; % QQQ
% ymax          = 5.3;



% *************************************************************************
% STEP 2: prepare variables------------------------------------------------
% step 2a: get variables from EwEResult structure -------------------------
biomass                         = EwEResult.Biomass;
consumption                     = EwEResult.Consumption; % end row is import consumption
production                      = EwEResult.Production;
TempFisheries                   = EwEResult.Landings + EwEResult.Discards;
% TL                              = EwEResult.TL_fisheries; % defines y-axis position; QQQ now replaced by supplied TL_position variable
% TL_fisheries                    = EwEResult.TL_fisheries;
EwE_number                      = EwEResult.EwE_code';
EwE_GroupType                   = EwEResult.GroupType;

% step 2b: get addresses of nutrients, fisheries, BA & EM (if they exist) -
looky_NUTRIENTS                 = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_ANYNitroNutr);
looky_BA                        = find(EwE_GroupType        == EwEResult.GroupTypeDef_BA);
looky_EM                        = find(EwE_GroupType        == EwEResult.GroupTypeDef_EM);
looky_fisheries                 = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_fishery); % take out fisheries code here, it will be replaced in proper position in code below
EwE_GroupType([looky_NUTRIENTS' looky_BA' looky_EM' looky_fisheries']) = [];
EwE_GroupType(end+1)            = EwEResult.GroupTypeDef_import; % add import diet to EwE_GroupType vector QQQ make sure group type ored here is same as in f_DietTrace

EwE_number([looky_NUTRIENTS' looky_BA' looky_EM' looky_fisheries'])    = [];





% *************************************************************************
% STEP 3: append import prey info to consumption---------------------------
EwE_GroupType(end+1)            = EwEResult.GroupTypeDef_import;    % add import diet to EwE_GroupType vector
import_consumption              = consumption(end, :);
import_size                     = sum(import_consumption, 2);       % size of import prey (t)
biomass(end+1)                  = import_size;
production(end+1)               = import_size;                      % How to represent import prey production size? Set equal to nansum(import_consumption)???
% TL(end+1)                       = 1.5;                              % import TL = 1.5, same as eggs
TempFisheries((end+1), :)       = 0;                                % no import fisheries allowed
consumption(:, (end+1))         = NaN;                              % append import as predator = all NaNs --- QQQ set to 0 OK?





% *************************************************************************
% STEP 4: add fisheries production-----------------------------------------
[fishery_rows, num_fisheries]               = size(TempFisheries);      % number of separate fisheries
EwE_GroupType((end+1):(end+num_fisheries))  = 5;                        % add Fisheries to EwE_GroupType vector
fishery_size                                = nansum(TempFisheries)';   % size of each fishery (t)
consumption(:, (end+1):(end+num_fisheries)) = TempFisheries;            % append fishery as predator columns
consumption((end+1):(end+num_fisheries), :) = 0;                        % append fishery as prey rows
biomass((end+1):(end+num_fisheries))        = fishery_size;
production((end+1):(end+num_fisheries))     = fishery_size;
% TL((end+1):(end+num_fisheries))             = TL_fisheries;





% *************************************************************************
% STEP 5: remove non-consumer predators (phyto, eggs, detritus) from diet columns
looky_phyto                         = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_ANYPrimaryProd);
looky_eggs                          = find(EwE_GroupType == EwEResult.GroupTypeDef_eggs);
looky_detritus                      = find(floor(EwE_GroupType) == EwEResult.GroupTypeDef_ANYDetritus);
NonConsumer_clms                    = [looky_phyto; looky_eggs; looky_detritus];
consumption(:, NonConsumer_clms)    = 0;
% TL(looky_detritus)                  = 1; % replace NaNs in TL vector (these are detritus groups)





% *************************************************************************
% STEP 6: PLOT WEB!--------------------------------------------------------
% step 6a: set up and initialize figure------------------------------------
[rows, clms]    = size(consumption);
linerange       = log((max(max(consumption)))+1);

fig1            = figure; hold on;
axis([xmin xmax ymin ymax])

% step 6b: plot gray link scafolding --------------------------------------
for loopROW = 1:rows
    for loopCLM = 1:clms
        current_line        = consumption(loopROW, loopCLM);
        linesize            = minline + ((maxline - minline)/(linerange))*log(current_line + 1);
        current_prey_x      = x_position(loopROW);
        current_prey_y      = TL_position(loopROW);
        current_predator_x  = x_position(loopCLM);
        current_predator_y  = TL_position(loopCLM);
        flow_color          = [0.6 0.6 0.6];
        % plot lines
        if ~isnan(current_line) && current_line >= minflow
            plot([current_prey_x current_predator_x], [current_prey_y current_predator_y], 'color', flow_color, 'linewidth', linesize);
        end
    end
end

% step 6c: plot FOOTPRINT -------------------------------------------------
% takes:
%       FootprintVector
%       DietTrace_downward

[Footprint_intensity, clm_order] = sort(FootprintVector);
if isempty(clm_order); clm_order = 1:clms; end

% plot links in reverse order of intensity so that gray lines do not hide colored lines        
for loopCLM = clm_order
	for loopROW = 1:rows
        current_line       = consumption(loopROW, loopCLM);
        linesize           = minline + ((maxline - minline)/(linerange))*log(current_line + 1);
        current_prey_x     = x_position(loopROW);
        current_prey_y     = TL_position(loopROW);
        current_predator_x = x_position(loopCLM);
        current_predator_y = TL_position(loopCLM);

        % set energy flow line color
        if ~isempty(DietTrace_downward)
        	current_trace = DietTrace_downward(loopROW, loopCLM).^LineIntensity; % trace fraction with increased color intensity
            % current_trace = FootprintVector(loopROW).^LineIntensity; % current_trace with increased color intensity
            
            if current_trace > 0
                flow_color = [(1-current_trace) 1 (1-current_trace)]; % color becomes more white as current_trace decreases
                flow_color = flow_color * grayness;
                % plot lines
                if ~isnan(current_line) && current_line >= minflow
                    plot([current_prey_x current_predator_x], [current_prey_y current_predator_y], 'color', flow_color, 'linewidth', linesize);
                end % (~isnan(current_line) && current_line >= minflow)
            end % (current_trace > 0)
        end % (~isempty(DietTrace_downward))
    end % (loopROW)
end % (loopCLM)

% step 6d: plot REACH -----------------------------------------------------
% takes:
%       TraceFraction_upward
%       DietTrace_upward

[Reach_intensity, row_order] = sort(TraceFraction_upward);
if isempty(row_order); row_order = 1:rows; end

% plot links in reverse order of intensity so that gray lines do not hide colored lines
for loopROW = row_order
	for loopCLM = 1:clms

        current_line       = consumption(loopROW, loopCLM);
        linesize           = minline + ((maxline - minline)/(linerange))*log(current_line + 1);
        current_prey_x     = x_position(loopROW);
        current_prey_y     = TL_position(loopROW);
        current_predator_x = x_position(loopCLM);
        current_predator_y = TL_position(loopCLM);

        % set energy flow line color
        if ~isempty(DietTrace_upward)
        	current_trace = DietTrace_upward(loopROW, loopCLM, 1).^LineIntensity; % trace fraction with increased color intensity
            % current_trace = FootprintVector(loopROW).^LineIntensity; % current_trace with increased color intensity
            
            if current_trace > 0
                flow_color = [1 (1-current_trace) (1-current_trace)]; % color becomes more white as current_trace decreases
                flow_color = flow_color * grayness;
                % plot lines
                if ~isnan(current_line) && current_line >= minflow
                    plot([current_prey_x current_predator_x], [current_prey_y current_predator_y], 'color', flow_color, 'linewidth', linesize);
                end
            end
        end
     end
end

% step 6e: plot group boxes -----------------------------------------------
% spotrange       = max(biomass);
spotrange       = max(production);
edgecolor       = [0.5 0.5 0.5];

if isempty(TraceFraction_upward)
	TraceFraction_upward    = zeros(1, rows);
end % (isempty(TraceFraction_upward))
    
if isempty(FootprintVector)
    FootprintVector         = zeros(1, rows);
end % (isempty(FootprintVector))


for loopROW = 1:rows
	% current_spot        = biomass(loopROW);
 	current_spot        = production(loopROW);
	spotsize            = minspot + ((maxspot - minspot)/(spotrange))*abs(current_spot);
    
    % set box color (plot either reach or footprint color for a given box based on which is the greater value)
	current_Reach       = TraceFraction_upward(loopROW);
	current_Footprint   = FootprintVector(loopROW);
    
    if current_Footprint > current_Reach
        current_BoxTrace    = current_Footprint.^BoxIntensity;                  % current_trace with increased color intensity
        spotcolor           = [(1-current_BoxTrace) 1 (1-current_BoxTrace)];    % color becomes more white as current_trace
    else
        current_BoxTrace    = current_Reach.^BoxIntensity;                      % current_trace with increased color intensity
        spotcolor           = [1 (1-current_BoxTrace) (1-current_BoxTrace)];    % color becomes more white as current_trace
    end % (current_Footprint > current_Reach)

    if ~isnan(x_position(loopROW)) && ~isnan(spotsize)
        % plot(loopROW, dat.TL(loopROW), 'o', 'markerfacecolor', spotcolor, 'markeredgecolor', edgecolor, 'markersize', spotsize)
        text(x_position(loopROW), TL_position(loopROW), GRPlabels{loopROW}, 'horizontalalignment', 'center', 'fontsize', spotsize, 'BackgroundColor', spotcolor, 'margin', 2, 'edgecolor', edgecolor)
    end % (~isnan(x_position(loopROW)) && ~isnan(spotsize))

end % (loopROW)

% step 6f: turn on for grid -----------------------------------------------
if strcmp(grid_switch, 'on')
    set(gca, 'xtick', [xmin:0.5:xmax], 'ytick', [ymin:0.5:ymax])
    grid on
    grid minor % QQQ
% else
%     axis off
end

% step 6g: set axis range and labels --------------------------------------
font_size = 12;
set(gca, 'ytick', [ymin 1:0.5:ymax])
set(gca, 'xticklabel', [], 'ticklength', [0 0], 'fontweight', 'bold', 'fontsize', font_size);
set(gca, 'yticklabel', [{''}, {'1.0'}, {'1.5'}, {'2.0'}, {'2.5'}, {'3.0'}, {'3.5'}, {'4.0'}, {'4.5'}, {'5.0'}, {'5.5'}]);
% set(gca, 'yticklabel', []);

ylabel('Trophic Level', 'fontweight', 'bold', 'fontsize', font_size)
% axis off




% end m-file***************************************************************