function [ConsumptionBudget_MonteCarlo, EnergyBudget_MonteCarlo] = f_E2E_MonteCarlo_08032018(MonteCarloConditions, ECOTRAN_PEDIGREE)
% Generate a stack of random, balanced ECOTRAN models.
% by Jim Ruzicka
%   Generates MonteCarlo Production Matrices and production estimates by drawing from a normal or a uniform distribution 
%   about each element of the EnergyBudget_matrix.
% calls:
%       none
% takes:
%       ECOTRAN_PEDIGREE (from f_E2E_pedigree_03082018)
%           ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix         CV values for each element of the ECOTRAN EnergyBudget_matrix)
%           ECOTRAN_PEDIGREE.CV_ConsumptionBudget           CV values for ConsumptionBudget terms, by row:
%                                                                                       feces
%                                                                                       metabolism
%                                                                                       eggs
%                                                                                       predation
%                                                                                       senescence
%                                                                                       BA
%                                                                                       EM
%           ECOTRAN_PEDIGREE.ECTRN_ConsumptionBudget        ECOTRAN consumption budget (as fraction of total consumption), by row:
%                                                                                       feces
%                                                                                       metabolism
%                                                                                       eggs
%                                                                                       predation
%                                                                                       senescence
%                                                                                       BA
%                                                                                       EM
%
%       MonteCarloConditions
%           MonteCarloConditions.EnergyBudget_matrix      (EM & BA rows already deleted; benthic detritus flowing to benthic detritus should be 0)
%           MonteCarloConditions.num_trials
%           MonteCarloConditions.pedigree
%           MonteCarloConditions.DistributionType         'normal' or 'uniform'
%
% returns:
%       EnergyBudget_MonteCarlo			(all Monte Carlo production arrays)
%       ConsumptionBudget_MonteCarlo	(all Monte Carlo Bioenergetic Budget arrays), by row:
%                                                                                       feces
%                                                                                       metabolism
%                                                                                       eggs
%                                                                                       predation
%                                                                                       senescence
%                                                                                       BA
%                                                                                       EM
%
%
% Part 1. Generate a random ConsumptionBudget matrix that defines, for each functional group, the
%           fraction of total consumption directed towards each of seven fates: feces, metabolism, 
%           eggs, predation, senescence, biomass accumulation, and emigration.
%         The distribution of uncertainties for each of the 7 terms is defined as variable 
%           CV_ConsumptionBudget in code f_E2E_pedigree_03222018.
%         The sum of each column of the ConsumptionBudget matrix (ConsumptionBudget_MonteCarlo)
%           sums to 1 to account for 100% of consumption. Feces, metabolism, and egg production 
%           are limited to values between 0 and 1 (actually held within a range slightly > 0 and 
%           slightly < 1 in order to not reduce predation and senescence to 0). Predation and senescence
%           may be any positive value >= 0. Biomass accumulation and emigration may be any value 
%           positive or negative. Feces values for fishing fleets represents discards. Emigration 
%           values for fishing fleets represent landings removed from the system so must be limited 
%           to values between 0 and 1.
%               ConsumptionBudget_MonteCarlo:
%                   row 1: feces        (0 <--> 1 only)
%                   row 2: metabolism   (0 <--> 1 only)
%                   row 3: eggs         (0 <--> 1 only)
%                   row 4: predation    (any positive value 0+, can be >1 because of BA & EM)
%                   row 5: senescence   (any positive value 0+, can be >1 because of BA & EM)
%                   row 6: BA           (any value)
%                   row 7: EM           (any value)     (fisheries: all cells 0 <--> 1 only)
%         Once a random ConsumptionBudget matrix is drawn, differences from 100% consumption are 
%           resolved:
%               1. The sum of feces, metabolism, and eggs kept within an overall maximum value.
%               2. Somatic production (predation and senescence) is adjusted to bring sum of all 
%                   intra-domain terms (feces, metabolism, eggs, predation, senescence) to 1. Thus, 
%                   total predation and senescence are limited by group physiology (feces, metabolism, 
%                   eggs) rather than by randomly drawing from an assumed level of uncertainty. However, 
%                   the scale of predation and senescence relative to each other are set by the
%                   assumed level of uncertainty in the pedigree.
%               3. If net growth (BA) and/or emigration (EM) terms are not zero, then somatic production 
%                   (predation and senescence) is raised or lowered to bring the sum of the consumption 
%                   budget to 1. Predation and senescence are kept at or above the minimum defined values. 
%                   Any imbalance to the system at this point (too much predation) is subtracted from 
%                   the BA term.
% Part 2. Generate a random ECOTRAN EnergyBudget matrix. 
%       - First, all terms within the random matrix are truncated to positive values only. 
%       - Second, the sum of ammonium terms are scaled to equal the metabolism in the Consumption Budget.
%       - Third, egg terms are scaled to equal the egg term in the Consumption Budget.
%       - Fourth, the sum of detritus terms are scaled to equal feces + senescence in the Consumption Budget.
%       - Fifth, the sum of predation terms are scaled to equal predation in the Consumption Budget
%       - Sixth, the defining "type" model is model 1 in the stack of Monte Carlo models
% NOTES:
%	- This does NOT contain any information about dfferent landing and discard ratios for individual groups 
%       that are caught. If needed, this could be defined downstream in the code for specific needs.
%	- Layer 1 of EnergyBudget_MonteCarlo is the defining "type" model.
%
% revision date: 8-3-2018


% *************************************************************************
% STEP 1: unpack variables (leave original variable structures unchanged)--
% step 1a: ----------------------------------------------------------------
EnergyBudget_matrix             = MonteCarloConditions.EnergyBudget_matrix(:, :, 1); % this must be a single 2D production matrix, choose top matrix just in case this is a stack of MonteCarlo models
num_trials                      = MonteCarloConditions.num_trials;                   % number of trials to run
DistributionType                = MonteCarloConditions.DistributionType;             % 'normal' or 'uniform'
[E2E_rows, E2E_clms]            = size(EnergyBudget_matrix);

% step 1b: ----------------------------------------------------------------
CV_EnergyBudget_matrix          = ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix;           % (2D matrix: E2E_rows X E2E_clms)
CV_ConsumptionBudget            = ECOTRAN_PEDIGREE.CV_ConsumptionBudget;             % rows: 1=feces; 2=metabolism; 3=eggs; 4=predation; 5=senescence; 6=BA; 7=EM
ECTRN_ConsumptionBudget         = ECOTRAN_PEDIGREE.ECTRN_ConsumptionBudget;          % rows: 1=feces; 2=metabolism; 3=eggs; 4=predation; 5=senescence; 6=BA; 7=EM

% step 1c: ECOTRAN addresses ----------------------------------------------
looky_AllNutrients              = ECOTRAN_PEDIGREE.looky_AllNutrients;               % technically, NO3 is not metabolism, but there is no flow from any group to NO3 in EnergyBudget_matrix so this wil not cause a problem
looky_detritus                  = ECOTRAN_PEDIGREE.looky_detritus;
looky_eggs                      = ECOTRAN_PEDIGREE.looky_eggs;
looky_predation                 = ECOTRAN_PEDIGREE.looky_predation;
looky_fishery                   = ECOTRAN_PEDIGREE.looky_fishery;
looky_PelagicDetritus           = ECOTRAN_PEDIGREE.looky_PelagicDetritus;
looky_BenthicDetritus           = ECOTRAN_PEDIGREE.looky_BenthicDetritus;

if exist('ECOTRAN_PEDIGREE.looky_ULTIMATEdetritus')
    looky_ULTIMATEdetritus	= ECOTRAN_PEDIGREE.looky_ULTIMATEdetritus;
else
    looky_ULTIMATEdetritus	= [];
end

num_nutrients                   = length(looky_AllNutrients);
num_eggs                        = length(looky_eggs);
num_detritus                    = length(looky_detritus);
num_predators                   = length(looky_predation);





% *************************************************************************
% STEP 2: prepare to create MonteCarlo arrays------------------------------
% step 2a: initialize result arrays ---------------------------------------
ConsumptionBudget_MonteCarlo        = zeros(7, E2E_clms, num_trials);
EnergyBudget_MonteCarlo            	= zeros(E2E_rows, E2E_clms, num_trials);
% Production_MonteCarlo           	  = zeros(E2E_clms, num_trials);

% step 2b: Zero-cell addresses of EnergyBudget & ConsumptionBudget --------
%          NOTE: entries are 0 where consumption budget in model are intentionally defined to be 0
EnergyBudget_ZeroScaler                                         = EnergyBudget_matrix;
EnergyBudget_ZeroScaler(EnergyBudget_ZeroScaler>0)              = 1;                        % (NOTE: all EnergyBudget_matrix cells are >=0)
ConsumptionBudget_ZeroScaler                                	= ECTRN_ConsumptionBudget;  % (2D matrix: 7 X E2E_clms)
ConsumptionBudget_ZeroScaler(ConsumptionBudget_ZeroScaler>0)   	= 1;                        % (2D matrix: 7 X E2E_clms)
ConsumptionBudget_ZeroScaler([1 7], looky_fishery)              = 1;                        % allow discards (feces) and landings (EM) for all fisheries; (2D matrix: 7 X E2E_clms)

% step 2c: assign minimum & maximum terms for ConsumptionBudget -----------
%          NOTE: SSS these are assumed values purposely set to unrealistically wide ranges
min_FECES               = 0.05;     % absolute minimum feces budget
min_MTBLSM              = 0.2;      % absolute minimum respiration budget
min_EGGS                = 0.005;    % absolute minimum egg budget
min_PREDATION           = 0.005;    % absolute minimum predation budget
min_SENESCENCE          = 0.005;    % absolute minimum senescence budget
min_DISCARDS            = 0.001; 	% absolute minimum fleet feces (discards) budget
min_LANDINGS            = 0.001;   	% absolute minimum fleet landings (EM) budget; NOTE: FFF could make this negative to simulate fish stocking programs

max_FECES               = 0.95;     % absolute maximum feces budget
max_MTBLSM              = 0.95;     % absolute minimum respiration budget
max_EGGS                = 0.95;     % absolute minimum egg budget
max_FME                 = 0.95;     % absolute maximum budget for FME (FME = feces + metabolism + eggs)
max_DISCARDS            = 1 - min_LANDINGS;    % absolute maximum fleet discards (feces) budget
max_LANDINGS            = 1 - min_DISCARDS;    % absolute maximum fleet landings (EM) budget


min_ConsumptionBudget   = [min_FECES; min_MTBLSM; min_EGGS; min_PREDATION; min_SENESCENCE; 1; 1];	% (vertical vector: 7 X 1)
min_ConsumptionBudget   = repmat(min_ConsumptionBudget, [1, E2E_clms]);                             % (2D matrix: 7 X E2E_clms)
min_ConsumptionBudget(1, looky_fishery) = min_DISCARDS;                                             % apply fishery discard minimum
min_ConsumptionBudget(7, looky_fishery) = min_LANDINGS;                                             % apply fishery landings minimum
min_ConsumptionBudget   = min_ConsumptionBudget .* ConsumptionBudget_ZeroScaler;                    % (2D matrix: 7 X E2E_clms)

max_ConsumptionBudget   = [max_FECES; max_MTBLSM; max_EGGS; 1; 1; 1; 1];                            % (vertical vector: 7 X 1)
max_ConsumptionBudget   = repmat(max_ConsumptionBudget, [1, E2E_clms]);                             % (2D matrix: 7 X E2E_clms)
max_ConsumptionBudget(1, looky_fishery) = max_DISCARDS;                                             % apply fishery discard maximum
max_ConsumptionBudget(7, looky_fishery) = max_LANDINGS;                                             % apply fishery landings maximum
max_ConsumptionBudget   = max_ConsumptionBudget .* ConsumptionBudget_ZeroScaler;                    % (2D matrix: 7 X E2E_clms)
max_FME                 = repmat(max_FME, [1, E2E_clms]);                                           % (horizontal vector: 1 X E2E_clms)
max_FME(looky_fishery)  = max_DISCARDS;                                                             % apply fishery discard maximum; (horizontal vector: 1 X E2E_clms)

% step 2d: minimum-cell addresses for MonteCarlo EnergyBudget -------------
min_EnergyBudget      	= 0.001 * EnergyBudget_matrix;                                              % SSS assumed minimum EnergyBudget values to prevent trace values from going to 0





% *************************************************************************
% STEP 3: prepare ConsumptionBudget for uniform distribution---------------
if strcmp(DistributionType, 'uniform')
    % step 3a: interval over which to draw random ConsumptionBudget -----------
    interval_low_ConsumptionBudget      = ECTRN_ConsumptionBudget - (ECTRN_ConsumptionBudget .* CV_ConsumptionBudget);
    interval_high_ConsumptionBudget     = ECTRN_ConsumptionBudget + (ECTRN_ConsumptionBudget .* CV_ConsumptionBudget);

    % step 3b: filter-out improper ConsumptionBudget interval values ----------
    %   row 1: feces        (0 <--> 1 only)
    %   row 2: metabolism   (0 <--> 1 only)
    %   row 3: eggs         (0 <--> 1 only)
    %   row 4: predation    (any positive value 0+, can be >1 because of BA & EM)
    %   row 5: senescence   (any positive value 0+, can be >1 because of BA & EM)
    %   row 6: BA           (any value)
    %   row 7: EM           (any value)     (fisheries: all cells 0 <--> 1 only)

    % filter out all negative values (except for BA & EM rows)
    CheckInterval_low                                    = interval_low_ConsumptionBudget(1:5, :);
    CheckInterval_high                                   = interval_high_ConsumptionBudget(1:5, :);
    CheckInterval_low(CheckInterval_low < 0)             = 0;
    CheckInterval_high(CheckInterval_high < 0)           = 0;
    interval_low_ConsumptionBudget(1:5, :)               = CheckInterval_low;
    interval_high_ConsumptionBudget(1:5, :)              = CheckInterval_high;

    % feces, metabolism, & eggs filter (no values > 1) (assumption that BA & EM terms are compensated for by ecology of individuals and not by changes to physiology)
    CheckInterval_low                                    = interval_low_ConsumptionBudget(1:3, :);
    CheckInterval_high                                   = interval_high_ConsumptionBudget(1:3, :);
    CheckInterval_low(CheckInterval_low > 1)             = 1;
    CheckInterval_high(CheckInterval_high > 1)           = 1;
    interval_low_ConsumptionBudget(1:3, :)               = CheckInterval_low;
    interval_high_ConsumptionBudget(1:3, :)              = CheckInterval_high;

    % % FFF set min and max parameter range--- metabolism (assumption that BA & EM terms are compensated for by ecology of individuals and not by changes to physiology)
    % CheckInterval_low                                   = interval_low_ConsumptionBudget(2, :);
    % CheckInterval_high                                  = interval_high_ConsumptionBudget(2, :);
    % CheckInterval_low(CheckInterval_low < min_MTBLSM)   = min_MTBLSM;
    % CheckInterval_high(CheckInterval_high < min_MTBLSM) = min_MTBLSM;
    % CheckInterval_low(CheckInterval_low > max_MTBLSM)   = max_MTBLSM;
    % CheckInterval_high(CheckInterval_high > max_MTBLSM) = max_MTBLSM;
    % interval_low_ConsumptionBudget(2, :)                = CheckInterval_low;
    % interval_high_ConsumptionBudget(2, :)               = CheckInterval_high;

    % fisheries group filter (EM = landings are always between 0 and 1)
    CheckInterval_low                                   = interval_low_ConsumptionBudget(7, looky_fishery);
    CheckInterval_high                                  = interval_high_ConsumptionBudget(7, looky_fishery);
    CheckInterval_low(CheckInterval_low < 0)            = 0;
    CheckInterval_high(CheckInterval_high < 0)          = 0;
    CheckInterval_low(CheckInterval_low > 1)            = 1;
    CheckInterval_high(CheckInterval_high > 1)          = 1;
    interval_low_ConsumptionBudget(7, looky_fishery)	= CheckInterval_low;
    interval_high_ConsumptionBudget(7, looky_fishery)	= CheckInterval_high;
    
end % if strcmp(DistributionType, 'uniform')





% *************************************************************************
% STEP 4: prepare EnergyBudget for uniform distribution--------------------
if strcmp(DistributionType, 'uniform')
    % step 4a: interval over which to draw random EnergyBudget_matrix ---------
    interval_low_EnergyBudget                                       = EnergyBudget_matrix - (EnergyBudget_matrix .* CV_EnergyBudget_matrix);
    interval_high_EnergyBudget                                      = EnergyBudget_matrix + (EnergyBudget_matrix .* CV_EnergyBudget_matrix);

    % step 4b: filter out improper EnergyBudget_matrix interval values --------
    %   metabolism                      (0 <--> 1 only)
    %   eggs                            (0 <--> 1 only)
    %   predation                       (any positive value 0+, can be >1 because of BA & EM)
    %   detritus (feces + senescence)   (any positive value 0+, can be >1 because of BA & EM)
    %   any fishery cell                (0 <--> 1 only) 

    % filter out all negative values (note that EnergyBudget_matrix does not have rows for BA and EM)
    interval_low_EnergyBudget(interval_low_EnergyBudget < 0)        = 0;
    interval_high_EnergyBudget(interval_high_EnergyBudget < 0)      = 0;

    % filter out metabolism & eggs >1
    CheckInterval_low                                               = interval_low_EnergyBudget([looky_AllNutrients; looky_eggs], :);
    CheckInterval_high                                              = interval_high_EnergyBudget([looky_AllNutrients; looky_eggs], :);
    CheckInterval_low(CheckInterval_low > 1)                        = 1;
    CheckInterval_high(CheckInterval_high > 1)                      = 1;
    interval_low_EnergyBudget([looky_AllNutrients; looky_eggs], :)	= CheckInterval_low;
    interval_high_EnergyBudget([looky_AllNutrients; looky_eggs], :)	= CheckInterval_high;

    % fisheries group filter, no values >1
    CheckInterval_low                                               = interval_low_EnergyBudget(:, looky_fishery);
    CheckInterval_high                                              = interval_high_EnergyBudget(:, looky_fishery);
    CheckInterval_low(CheckInterval_low > 1)                        = 1;
    CheckInterval_high(CheckInterval_high > 1)                      = 1;
    interval_low_EnergyBudget(:, looky_fishery)                     = CheckInterval_low;
    interval_high_EnergyBudget(:, looky_fishery)                    = CheckInterval_high;

end % if strcmp(DistributionType, 'uniform')





% *************************************************************************
% STEP 5: generate random models and production vectors--------------------
for trial_loop = 2:num_trials
    
    % step 5a: generate a random ConsumptionBudget ------------------------
    %   row 1: feces
    %   row 2: metabolism
    %   row 3: eggs
    %   row 4: predation
    %   row 5: senescence
    %   row 6: BA
    %   row 7: EM           (NOTE: a negative value is an input into the domain)
    
    
    if strcmp(DistributionType, 'uniform')
        % for uniform distribution
        random_matrix                         = rand(7, E2E_clms); % draw from UNIFORM distribution
        MonteCarloArray                       = interval_low_ConsumptionBudget + ((interval_high_ConsumptionBudget - interval_low_ConsumptionBudget) .* random_matrix); % produce the new random ConsumptionBudget
    else
        % for normal distribution
        random_matrix                         = randn(7, E2E_clms); % draw from NORMAL distribution
        MonteCarloArray                       = ECTRN_ConsumptionBudget + ((ECTRN_ConsumptionBudget .* CV_ConsumptionBudget) .* random_matrix);
    end % if strcmp(DistributionType, 'uniform')

    
    % A) truncate all below-minimum values (except for BA & EM rows)
      CheckInterval                             = MonteCarloArray(1:5, :);
      MinInterval                               = min_ConsumptionBudget(1:5, :);
      CompInterval                           	= CheckInterval - MinInterval;
      looky_TooSmall                            = find(CompInterval < 0);
      CheckInterval(looky_TooSmall)             = MinInterval(looky_TooSmall);
      MonteCarloArray(1:5, :)                   = CheckInterval;
      
    % B) truncate all above-maximum feces, metabolism, & egg values
      CheckInterval                             = MonteCarloArray(1:3, :);
      MaxInterval                               = max_ConsumptionBudget(1:3, :);
      CompInterval                           	= MaxInterval - CheckInterval;
      looky_TooBig                              = find(CompInterval < 0);
      CheckInterval(looky_TooBig)               = MaxInterval(looky_TooBig);
      MonteCarloArray(1:3, :)                   = CheckInterval;
      
    % C) truncate FME (feces + metabolism + eggs); find FME > max_FME and renormalize FME to max_FME but obey minimum standards
      CheckInterval                             = MonteCarloArray(1:3, :);          % feces, metabolism, and egg terms; (2D matrix: 3 X E2E_clms)
      MinInterval                               = min_ConsumptionBudget(1:3, :);    % minimum allowed value for each term (feces, metabolism, eggs)
      dfrInternal                               = CheckInterval - MinInterval;      % amount that each term (feces, metabolism, eggs) is above minimum allowed value and can be changed
      total_FME                                 = sum(CheckInterval);               % (horizontal vector: 1 X E2E_clms)
      dfr_FME                                   = (max_FME - total_FME);            % amount above max_FME that must be reduced; positive values set to 0; (horizontal vector: 1 X E2E_clms)
      looky_positive                            = find(dfr_FME > 0);
      dfr_FME(looky_positive)                   = 0;
      total_FME                                 = sum(dfrInternal);                 % recalculate using dfrInternal; (horizontal vector: 1 X E2E_clms)
      total_FME                                 = repmat(total_FME, [3, 1]);        % (2D matrix: 3 X E2E_clms)
      dfr_FME                                   = repmat(dfr_FME, [3, 1]);          % (2D matrix: 3 X E2E_clms)
      adjusted_FME                              = CheckInterval + (dfrInternal ./ total_FME) .* dfr_FME; % (2D matrix: 3 X E2E_clms)
      adjusted_FME(isnan(adjusted_FME))         = 0;                                % set div/0 NaNs to 0; (2D matrix: 3 X E2E_clms)
      
      % correct for cases where all FME terms are below minimum values
      total_FME                                 = sum(adjusted_FME);                % (horizontal vector: 1 X E2E_clms)
      looky_zero                                = find(total_FME == 0);             % these are the clms where either all FME terms are below minimu values or that FME terms are not defined (e.g., phytoplankton and fleets)
      FME_fix                                   = zeros(1, E2E_clms);               % (horizontal vector: 1 X E2E_clms)
      FME_fix(looky_zero)                       = 1;
      FME_fix                                   = repmat(FME_fix, [3, 1]);          % (2D matrix: 3 X E2E_clms)
      FME_fix                                   = FME_fix .* ConsumptionBudget_ZeroScaler(1:3, :);	% zero-out cases where feces, metabolism, or eggs are defined as zero in the model
      FME_fix                                   = FME_fix .* MinInterval;           % replace any missing values with minimum values
      adjusted_FME                              = adjusted_FME + FME_fix;
      MonteCarloArray(1:3, :)                   = adjusted_FME;
      
    % D) adjust somatic production (predation and senescence) to bring sum of intra-domain parameters (feces, metabolism, eggs, predation, senescence) to 1
      CheckInterval                             = MonteCarloArray(4:5, :);          % predation and senescence terms; (2D matrix: 2 X E2E_clms)
      total_PS                                  = sum(CheckInterval);               % current sum of predation and senescence terms; (horizontal vector: 1 X E2E_clms)
      total_PS                                  = repmat(total_PS, [2, 1]);         % (2D matrix: 2 X E2E_clms)
      total_FME                                 = sum(MonteCarloArray(1:3, :));     % sum of FME terms (feces, metabolism, eggs); (horizontal vector: 1 X E2E_clms)
      dfr_PS                                    = 1 - total_FME;                    % value for sum of PS
      dfr_PS                                    = repmat(dfr_PS, [2, 1]);           % (2D matrix: 2 X E2E_clms)
      adjusted_PS                               = (CheckInterval ./ total_PS) .* dfr_PS;	% (2D matrix: 2 X E2E_clms)
      adjusted_PS(isnan(adjusted_PS))           = 0;                                % set div/0 NaNs to 0 (e.g., fleets); (2D matrix: 2 X E2E_clms)
      MonteCarloArray(4:5, :)                   = adjusted_PS;

    % E) balance net growth and emigration (BA + EM) with somatic production (predation and senescence)
      total_PS                                  = sum(MonteCarloArray(4:5, :));  	% total somatic production; (horizontal vector: 1 X E2E_clms);
      total_BAEM                                = sum(MonteCarloArray(6:7, :));    	% total BA and EM (biomass accumulation and emigration); (horizontal vector: 1 X E2E_clms); 
      pred_fraction                             = MonteCarloArray(4, :) ./ total_PS;	% fraction of somatic production going to predation
      pred_fraction(isnan(pred_fraction))       = 0;                             	% set div/0 NaNs to 0
      scen_fraction                             = 1 - pred_fraction;               	% fraction of somatic production going to senescence
      new_PS                                    = total_PS - total_BAEM;            % subtract net BAEM from total somatic production (predation and senescence); (horizontal vector: 1 X E2E_clms)
      
      % filter out instances of somatic production (= predation + senescence) being reduced below a minimum assumed value
      min_PS                                    = 0.1 * total_PS;                	% SSS assumed minimum somatic production can only be cut by 90% to accomodate BA + EM; (horizontal vector: 1 X E2E_clms)
      looky_TooSmall                            = find(new_PS < min_PS);
      new_PS(looky_TooSmall)                    = min_PS(looky_TooSmall);
      
      % distribute new_PS between predation and senescence and enforce defined minimum P and S values
      adjusted_P                                = new_PS .* pred_fraction;          % (horizontal vector: 1 X E2E_clms)
      adjusted_S                                = new_PS .* scen_fraction;          % (horizontal vector: 1 X E2E_clms)
      min_P                                     = min_ConsumptionBudget(4, :);      % minimum predation term with defined zero terms; (horizontal vector: 1 X E2E_clms)
      CompInterval_P                            = adjusted_P - min_P;               % (horizontal vector: 1 X E2E_clms)
      looky_TooSmall                            = find(CompInterval_P < 0);         % clm addresses of predation terms that are less than the defined minimum
      adjusted_P(looky_TooSmall)                = min_P(looky_TooSmall);
      adjusted_S(looky_TooSmall)                = adjusted_S(looky_TooSmall) + CompInterval_P(looky_TooSmall); % reduce senescence by amount predation was raised
      min_S                                     = min_ConsumptionBudget(5, :);      % minimum senescence term with defined zero terms; (horizontal vector: 1 X E2E_clms)
      CompInterval_S                            = adjusted_S - min_S;               % (horizontal vector: 1 X E2E_clms)
      looky_TooSmall                            = find(CompInterval_S < 0);         % clm addresses of predation terms that are less than the defined minimum
      adjusted_S(looky_TooSmall)                = min_S(looky_TooSmall);
      adjusted_P(looky_TooSmall)                = adjusted_P(looky_TooSmall) + CompInterval_S(looky_TooSmall); % reduce predation by amount senescence was raised
      MonteCarloArray(4, :)                     = adjusted_P;
      MonteCarloArray(5, :)                     = adjusted_S;
      
    % F) put remaining imbalance onto BA
      sum_MonteCarloArray                       = sum(MonteCarloArray);           	% (horizontal vector: 1 X E2E_clms);
      imbalance                                 = sum_MonteCarloArray - 1;       	% positive imbalance means too much export, take from BA
      MonteCarloArray(6, :)                     = MonteCarloArray(6, :) - imbalance;
      
    % G) fisheries landings (discards were brought into range above)
      MonteCarloArray(6, looky_fishery)         = 0;                                % set BA for fleets to 0
      MonteCarloArray(7, looky_fishery)         = 1 - MonteCarloArray(1, looky_fishery); % landings (EM) are defined by the random draw of discards
      
      
	MonteCarloArray                         = MonteCarloArray .* ConsumptionBudget_ZeroScaler;	% this step cleans up tiny rounding error in BA terms and makes sure we didn't create BA and EM terms where none originally existed
    
    total_MonteCarloArray                   = sum(MonteCarloArray);
    total_MonteCarloArray                   = repmat(total_MonteCarloArray, [7, 1]);
    MonteCarloArray                         = MonteCarloArray ./ total_MonteCarloArray;        	% final normalization set to make sure MonteCarlo_ConsumptionBudget clms all sum to 1; may bring some terms slightly out of the defined min & max range
    MonteCarloArray(isnan(MonteCarloArray)) = 0;                                                % set div/0 NaNs to 0
    MonteCarlo_ConsumptionBudget            = MonteCarloArray;
      
      
    % step 5b: generate a random EnergyBudget -----------------------------
    if strcmp(DistributionType, 'uniform')
        % for uniform distribution
        random_matrix                         = rand(E2E_rows, E2E_clms); % draw from a UNIFORM distribution
        MonteCarloArray                       = interval_low_EnergyBudget + ((interval_high_EnergyBudget - interval_low_EnergyBudget) .* random_matrix); % produce the new random EnergyBudget_matrix    
    else
        % for normal distribution
        random_matrix                         = randn(E2E_rows, E2E_clms); % draw from a NORMAL distribution
        MonteCarloArray                       = EnergyBudget_matrix + ((EnergyBudget_matrix .* CV_EnergyBudget_matrix) .* random_matrix);
    end % if strcmp(DistributionType, 'uniform')
    
    
    % A) filter out negative values
      looky_TooSmall                            = find(MonteCarloArray < 0);
      MonteCarloArray(looky_TooSmall)           = min_EnergyBudget(looky_TooSmall);
    
    % B) normalize metabolism terms to ConsumptionBudget value
	  MonteCarlo_MTBLSM                         = sum(MonteCarloArray(looky_AllNutrients, :));
      scaler_MTBLSM                             = MonteCarlo_ConsumptionBudget(2, :) ./ MonteCarlo_MTBLSM;
      scaler_MTBLSM(isnan(scaler_MTBLSM))       = 0; % filter div/0 NaNs
      MonteCarloArray(looky_AllNutrients, :)    = MonteCarloArray(looky_AllNutrients, :) .* repmat(scaler_MTBLSM, [num_nutrients 1]);
    
    % C) normalize egg terms to ConsumptionBudget value
      MonteCarlo_eggs                           = sum(MonteCarloArray(looky_eggs, :));
      scaler_eggs                               = MonteCarlo_ConsumptionBudget(3, :) ./ MonteCarlo_eggs;
      scaler_eggs(isnan(scaler_eggs))           = 0; % filter div/0 NaNs
      MonteCarloArray(looky_eggs, :)            = MonteCarloArray(looky_eggs, :) .* repmat(scaler_eggs, [num_eggs 1]);
    
    % D) normalize detritus terms to ConsumptionBudget value (feces + senescence)
      MonteCarlo_detritus                       = sum(MonteCarloArray(looky_detritus, :));
      scaler_detritus                           = sum(MonteCarlo_ConsumptionBudget([1 5], :)) ./ MonteCarlo_detritus;
      scaler_detritus(isnan(scaler_detritus))   = 0; % filter div/0 NaNs
      scaler_detritus(isinf(scaler_detritus))   = 0; % filter div/0 NaNs
      MonteCarloArray(looky_detritus, :)        = MonteCarloArray(looky_detritus, :) .* repmat(scaler_detritus, [num_detritus 1]);
      
    % E) normalize predation terms to ConsumptionBudget value
      % NOTE: special case for terminal benthic detritus group (no senescence in EnergyBudget_matrix), increase predation to compensate
      if isempty(looky_ULTIMATEdetritus)
        
            tempMonteCarlo_ConsumptionBudget       	  = MonteCarlo_ConsumptionBudget;
            tempMonteCarlo_ConsumptionBudget(4, looky_BenthicDetritus) = sum(MonteCarlo_ConsumptionBudget([4 5], looky_BenthicDetritus)); % add senescence to predation for terminal benthic detritus group
            tempMonteCarlo_ConsumptionBudget(5, looky_BenthicDetritus) = 0;
            MonteCarlo_predation                      = sum(MonteCarloArray(looky_predation, :));
            scaler_predation                          = tempMonteCarlo_ConsumptionBudget(4, :) ./ MonteCarlo_predation;
            scaler_predation(isnan(scaler_predation)) = 0; % filter div/0 NaNs
            MonteCarloArray(looky_predation, :)       = MonteCarloArray(looky_predation, :) .* repmat(scaler_predation, [num_predators 1]); % (2D matrix: num_grps X num_grps)
      
      else

            MonteCarlo_predation                      = sum(MonteCarloArray(looky_predation, :));
            scaler_predation                          = MonteCarlo_ConsumptionBudget(4, :) ./ MonteCarlo_predation;
            scaler_predation(isnan(scaler_predation)) = 0; % filter div/0 NaNs
            MonteCarloArray(looky_predation, :)       = MonteCarloArray(looky_predation, :) .* repmat(scaler_predation, [num_predators 1]); % (2D matrix: num_grps X num_grps)
          
      end
      
      
    MonteCarlo_EnergyBudget                     = MonteCarloArray;

    
    % step 5c: store results ----------------------------------------------
    ConsumptionBudget_MonteCarlo(1:7, :, trial_loop)    = MonteCarlo_ConsumptionBudget;
    EnergyBudget_MonteCarlo(:, :, trial_loop)           = MonteCarlo_EnergyBudget;
        
end % (trial_loop)





% *************************************************************************
% STEP 6: store defining "type" model in layer 1---------------------------
EnergyBudget_MonteCarlo(:, :, 1)            = EnergyBudget_matrix;      % store defining "type" model in layer 1
ConsumptionBudget_MonteCarlo(:, :, 1)       = ECTRN_ConsumptionBudget;  % store defining "type" model in layer 1


% end m-file***************************************************************