function [ECOTRANphysics] = f_ECOTRANphysics_downwelling_08022018(PHYSICSinput)
% return Advection and Mixing exchanges between boxes
% use for CGoA downwelling setting
% takes:
%       internally defined time-series of advection & mixing rates
% returns ECOTRANphysics:
%       AdvectionMatrix         advection rate = volume transported per day; (m3/d)
%                               (3D matrix: time X  source box X destination box)
%                               (values can be positive or negative)
%       VerticalMixMatrix       vertical mixing rate = volume mixed per day; (m3/d)
%                               (3D matrix: time X  source box X destination box)
%                               (values can be positive or negative; sign defines direction of net mixing)
% calls:
%       round2                  (round to a specified number of decimal places)
%       cgoa_ancyc.mat          (dataset with monthly mean salinity, temperature, and NO3+NO2 data)
%       f_LightIntensity
% NOTE: What happens in cases when there is a complete advective washout of a spatial box?)
%       I added cap to advection rate to prevent >100% box washout (but do not account for river flux)
%       Added warning for days when AdvectionMatrix > BoxVolume; see ECOTRANphysics.BoxWashoutTime
% revision date: 8-2-2018


% *************************************************************************
% STEP 1: basic domain parameters (CGoA downwelling)-----------------------
% step 1a: physical parameters --------------------------------------------
latitude     = 57;                      % middle of domain
rho_water    = 1026;                    % seawater density; (kg/m3)
rho_air      = 1.220;                   % air density (kg/m3) (Schwing et al. 1996)
coriollis    = 1.26 * 10^(-4);          % coriollis factor (1/s)
midmonth_doy = [15 45 74 105 135 166 196 227 258 288 319 349]; % mid-month day-of-year

% step 1b: time domain and vectors ----------------------------------------
datestart    = PHYSICSinput.datestart;  % enter starting date
dateend      = PHYSICSinput.dateend;    % enter ending date
dt           = PHYSICSinput.dt;         % t-step (d)
t_grid       = PHYSICSinput.t_grid;     % t_grid; (vertical vector)
num_t        = length(t_grid);          % length of t_grid; (scaler)
min_t        = min(t_grid);
max_t        = max(t_grid);
num_years    = ceil(max_t/365);

% step 1c: Box ID info ----------------------------------------------------
num_boxes                   = 5;        % I, II, III, IV, V
num_AdvecFluxes             = 6;        % advective fluxes: A_12, A_13, A_24, A_35, A_4offshore, A_5offshore
num_MixFluxes               = 2;        % vertical mix fluxes: w_23, w_45
num_HorizMixFluxes          = 6;        % horizontal mix fluxes: u_12 u_13 u_24 u_35 u_4ocean u_5ocean
looky_SubSurfaceBoxes       = [3, 5];
looky_SurfaceBoxes          = [2, 4];
looky_OffshoreSurfaceBox    = 4;
looky_OffshoreSubSurfaceBox = 5;
looky_RiverFluxBox          = 2;        % River input from alongshore; (Box 2 in CGoA downwelling model)

% step 1d: spatial dimensions ---------------------------------------------
%          NOTE: Mixed Layer Depth (only used for light intensity calculations, not used for advection rate calculations)
%          NOTE: box geometry with slanty bottom
maxMLD                      = 15;                                                       % max MLD; (m) % alt 40
minMLD                      = 15;                                                       % min MLD; (m) % alt 10
rangeMLD                    = maxMLD - minMLD;                                          % set minimum mixed layer depth and annual range of the mixed layer depth; (m)
MLD                         = (cos(((t_grid/365)*pi)*2) * (rangeMLD * 0.5) + (minMLD + rangeMLD * 0.5)); % MLD; (m); (vertical vector: time X 1)
BoxLength                   = [15000, 75000, 75000, 20000, 20000];                      % (m); I, II, III, IV, V
BoxLength                   = repmat(BoxLength, [num_t, 1]);                            % (m); (2D matrix: time X num_boxes)
BoxWidth                    = [1, 1, 1, 1, 1];                                          % (m); I, II, III, IV, V
BoxWidth                    = repmat(BoxWidth, [num_t, 1]);                             % (m); (2D matrix: time X num_boxes)
BoxHeight_inshore           = [repmat(0,  [num_t, 1]), MLD, (60-MLD),  MLD, (200-MLD)]; % (m); (2D matrix: time X num_boxes); I, II, III, IV, V
BoxHeight_offshore          = [repmat(60, [num_t, 1]), MLD, (200-MLD), MLD, (300-MLD)]; % (m); (2D matrix: time X num_boxes); I, II, III, IV, V
BoxHeight                   = 0.5 * (BoxHeight_inshore + BoxHeight_offshore);           % mean box height; (m); (2D matrix: time X num_boxes); I, II, III, IV, V
BoxArea                     = BoxHeight .* BoxLength;                                   % (m2); (2D matrix: time X num_boxes); I, II, III, IV, V
BoxVolume                   = BoxArea .* BoxWidth;                                      % I, II, III, IV, V; (m3); (2D matrix: time X num_boxes);





% *************************************************************************
% STEP 2: prepare advection & mixing parameters----------------------------
% step 2a: Brink Physics raw values (CGoA upwelling) ----------------------
% NOTE: ONshore direction is positive

% Wind-driven advection ---------------------------------------------------
% wind-driven advection (Qw); Qw = tau .* (1/coriollis) .* (1/rho_water); daily advective flux rate; (m3/s per 1m BoxWidth)
% Alongshore wind stress (tau) monthly mean: (N/m^2) = (kg*m/m2/s2) = (kg/m/s2)
% tau = [???, ???, ???, ???, ???, ???, ???, ???, ???, ???, ???, ???]; % wind stress, monthly mean: (N/m^2) = (kg*m/m2/s2) = (kg/m/s2)
tau = []; % (not used for CGoA)
% Qw  = tau_doy .* (1/coriollis) .* (1/1000) * (-1); % daily advective flux rate; (m3/s per 1m BoxWidth); (vertical vector); (NOTE: change of sign to make offshore "POSITIVE")
Qw = [-0.868705005940316, -0.676948832267787, -0.722799188188498, -0.526499940876143, -0.217798334645914, -0.121521419624197, -0.126884526751927, -0.295377695651937, -0.621209552418220, -0.603983821205372, -0.785774853445138, -1.01722141644584];
Qw = repmat(Qw, [6, 1]) .* repmat([(-1); (1); (-1); (1); (-1); (1)], [1, 12]); % (2D matrix: num_AdvecFluxes X 12 months); Advection Fluxes: A_12 A_13 A_24 A_35 A_4offshore A_5offshore; (m3/s per 1m BoxWidth)
% NOTE: OFFshore direction is negative for Qw definition

% river infux (Qr); (m3/s per 1m BoxWidth) --------------------------------
% NOTE: enters Box II
% Qr = [NaN, NaN, 0.0199524817600028, 0.0108016334304876, 0.00377081329425930, NaN, 0.00460470845498319, 0.0199441242674923, 0.0419716078728665, 0.0299303146912406, NaN, 0.0237018206706539];
Qr = [0.022410382, 0.021160602, 0.0199524817600028, 0.0108016334304876, 0.00377081329425930, 0.00460470845498319, 0.00460470845498319, 0.0199441242674923, 0.0419716078728665, 0.0299303146912406, 0.026816068, 0.0237018206706539]; % NOTE: Jan, Feb, June, and November values made up from NaNs by linear interpolation
Qr = repmat(Qr, [6, 1]) .* repmat([(0); (0); (-1); (0); (-1); (0)], [1, 12]); % (2D matrix: num_AdvecFluxes X 12 months); Advection Fluxes: A_12 A_13 A_24 A_35 A_4offshore A_5offshore; (m3/s per 1m BoxWidth)

% NOTE: ONshore direction is positive. 
%       So Qw values are all given as negative, and are oriented offshore. 
%       But, Q_24 = f(-Qw), so Qw contribution Q_42 is oriented ONshore. 
%       Also Q_42 = f(-Qr) and Qr > 0, so Qr contribution in Q_42 is negative and is oriented OFFshore.

% horizontal mixing rate, monthly mean: (m2/s) ----------------------------
%       NOTE difference from GB bank units which were (m3/s per 1m BoxWidth)
%       NOTE: ue flux already accounts for box height
%       NOTE: NaN values are filled in as zeros (can use vertical interpolation, but I don't have formula Ken used so I dont know how to conserve volume within boxes to use anything other than 0)

% ue = [
%         0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_12
%         NaN NaN 0.00520755 0 0.014342343 NaN 0.008161398 0.013032939 0.044879593 0.00466496 NaN 0               % u_13
%         0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_24
%         NaN NaN 0.008972708 0.012116005 0.003593215 NaN 0.007840667 0.009645155 0.024557155 0.006638493 NaN 0   % u_35
%         0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_4ocean
%         0 0 0 0 0 0 0 0 0 0 0 0];                                                                               % u_5ocean

ue = [
        0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_12
        0 0 0.00520755 0 0.014342343 0 0.008161398 0.013032939 0.044879593 0.00466496 0 0                       % u_13
        0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_24
        0 0 0.008972708 0.012116005 0.003593215 0 0.007840667 0.009645155 0.024557155 0.006638493 0 0           % u_35
        0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_4ocean
        0 0 0 0 0 0 0 0 0 0 0 0                                                                                 % u_5ocean
      ];        % (2D matrix: num_AdvecFluxes X 12 months); (m2/s)

% vertical mixing rate, monthly mean: (m/s) -------------------------------
% we      = [NaN NaN 0 0 3.06348E-07 NaN 0 0 1.32085E-06 0 NaN 0];	  % (m/s); calculated from Ken's original values of L2*we where L2 = 50000m
we      = [0 0 0 0 3.06348E-07 0 0 0 1.32085E-06 0 0 0];            % (m/s); NOTE: Jan, Feb, June, and November values made up from NaNs for debugging; calculated from Ken's original values of L2*we where L2 = 50000m
we      = repmat(we, [num_MixFluxes, 1]);                           % (2D matrix: num_MixFluxes X 12 months); Advection Fluxes: w_23 w_45; (m/s)

% allow for cyclical repeats over several years ---------------------------
build_midmonth_doy = [];
% build_tau          = [];
build_Qw           = [];
build_Qr           = [];
build_we           = [];
build_ue           = [];
for year_loop = 1:num_years
    build_midmonth_doy = [build_midmonth_doy (midmonth_doy + (365 * (year_loop - 1)))];
%     build_tau          = [build_tau tau];
    build_Qw           = [build_Qw Qw]; % (2D matrix: num_AdvecFluxes X months); (m3/s per 1m BoxWidth)
    build_Qr           = [build_Qr Qr]; % (2D matrix: num_AdvecFluxes X months); (m3/s per 1m BoxWidth)
    build_we           = [build_we we]; % (2D matrix: num_MixFluxes   X months); (m/s)
    build_ue           = [build_ue ue]; % (2D matrix: num_AdvecFluxes X months); (m2/s)
end

% step 2b: interpolate monthly means to daily values ----------------------
% tau_doy      = interp1([1 build_midmonth_doy (365*num_years)], [mean(tau([1 12])) build_tau mean(tau([1 12]))], t_grid, 'spline'); % (vertical vector)
tau_doy      = []; % (not used for CGoA)
Qw_doy       = interp1([1 build_midmonth_doy (365*num_years)]', [mean(Qw(:, [1 12]), 2) build_Qw mean(Qw(:, [1 12]), 2)]', t_grid', 'spline'); % (2D matrix: num_t X num_AdvecFluxes); (m3/s per 1m BoxWidth)
Qr_doy       = interp1([1 build_midmonth_doy (365*num_years)]', [mean(Qr(:, [1 12]), 2) build_Qr mean(Qr(:, [1 12]), 2)]', t_grid', 'spline'); % (2D matrix: num_t X num_AdvecFluxes); (m3/s per 1m BoxWidth)
we_doy       = interp1([1 build_midmonth_doy (365*num_years)]', [mean(we(:, [1 12]), 2) build_we mean(we(:, [1 12]), 2)]', t_grid', 'spline'); % (2D matrix: num_t X num_MixFluxes);   (m/s)
% we_doy       = we_doy / 2; % NOTE: division by 1/2 already accounted for in original calculation of rates; vertical mixing extends 1/2 into top box & 1/2 into bottom box
ue_doy       = interp1([1 build_midmonth_doy (365*num_years)]', [mean(ue(:, [1 12]), 2) build_ue mean(ue(:, [1 12]), 2)]', t_grid', 'spline'); % (2D matrix: num_t X num_AdvecFluxes); (m2/s); NOTE transpose

Qr_doy(:, 6)       = Qr_doy(:, 6) * 0; % for CGoA model only, filter out very tiny Qr_doy values in clm 6 created during spline interpolation
we_doy(we_doy < 0) = 0;                % filter out negative vertical mixing (we) values created during spline interpolation;   (2D matrix: num_t X num_AdvecFluxes); (m/s)
ue_doy(ue_doy < 0) = 0;                % filter out negative horizontal mixing (ue) values created during spline interpolation; (2D matrix: num_t X num_AdvecFluxes); (m2/s)

% step 2c: put a limit on AdvecFluxRate and RiverFluxRate to prevent total washout of small boxes on extreme days
%          NOTE: still possible to have box washed out if RiverFluxRate tips the balance
max_AdvecFluxRate                = min((BoxVolume * 1/(60*60*24)), [], 2); % max absolute advection rate (m3/s) = one BoxVolume per day; (vertical vector)
max_AdvecFluxRate                = repmat(max_AdvecFluxRate, [1, num_AdvecFluxes]);
looky_WashoutTime                = find(abs(Qw_doy) > max_AdvecFluxRate);
Qw_doy(looky_WashoutTime)        = sign(Qw_doy(looky_WashoutTime)) .* max_AdvecFluxRate(looky_WashoutTime);
looky_WashoutTime                = find(abs(Qr_doy) > max_AdvecFluxRate);
Qr_doy(looky_WashoutTime)        = sign(Qr_doy(looky_WashoutTime)) .* max_AdvecFluxRate(looky_WashoutTime);





% *************************************************************************
% STEP 3: define Advective fluxes and River fluxes to use for TimeSeries---
% step 3a:  ---------------------------------------------------------------
Advection_TimeSeries        = Qw_doy; % (2D matrix: num_t X num_AdvecFluxes);   (m3/s per 1m BoxWidth); Advection Fluxes: A_12 A_13 A_24 A_35 A_4offshore A_5offshore; NOTE: Qw & (Qw + Qr) always < 0; NOTE: onshore is POSITIVE, offshore is NEGATIVE
RiverFlux_TimeSeries        = Qr_doy; % (2D matrix: num_t X num_AdvecFluxes);   (m3/s per 1m BoxWidth); Advection Fluxes: A_12 A_13 A_24 A_35 A_4offshore A_5offshore; NOTE: (Qw + Qr) always < 0;      NOTE: onshore is POSITIVE, offshore is NEGATIVE
VerticalMixing_TimeSeries   = we_doy; % (2D matrix: time X num_MixFluxes);      (m/s);                  Mixing Fluxes:    w_23 w_45;                                   NOTE: all POSITIVE & already divided 1/2 upward and 1/2 downward
HorizontalMixing_TimeSeries = ue_doy; % (2D matrix: time X num_HorizMixFluxes); (m2/s);                 Mixing Fluxes:    u_12 u_13 u_24 u_35 u_4ocean u_5ocean;       NOTE: all POSITIVE & already divided 1/2 shoreward and 1/2 oceanward





% *************************************************************************
% STEP 4: Physical Structure (CGoA downwelling)----------------------------
% step 4a: define advective connections between boxes (3D matrix: destination boxes X source boxes X specific advection flux)
%          NOTE: the sign is used to identify source box depending on onshore-vs-offshore flux
AdvcDirectionSwitch(1:5, 1:5, 1) = [
-1	1	0	0	0
-1	1	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % A_12 (matrix: spatial cell X spatial cell: I II III IV V)

AdvcDirectionSwitch(1:5, 1:5, 2) = [
-1	0	1	0	0
0	0	0	0	0
-1	0	1	0	0
0	0	0	0	0
0	0	0	0	0]; % A_13 (matrix: spatial cell X spatial cell: I II III IV V)

AdvcDirectionSwitch(1:5, 1:5, 3) = [
0	0	0	0	0
0	-1	0	1	0
0	0	0	0	0
0	-1	0	1	0
0	0	0	0	0]; % A_24 (matrix: spatial cell X spatial cell: I II III IV V)

AdvcDirectionSwitch(1:5, 1:5, 4) = [
0	0	0	0	0
0	0	0	0	0
0	0	-1	0	1
0	0	0	0	0
0	0	-1	0	1]; % A_35 (matrix: spatial cell X spatial cell: I II III IV V)

AdvcDirectionSwitch(1:5, 1:5, 5) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	-1	0
0	0	0	0	0]; % A_4ocean (matrix: spatial cell X spatial cell: I II III IV V)

AdvcDirectionSwitch(1:5, 1:5, 6) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	-1]; % A_5ocean (matrix: spatial cell X spatial cell: I II III IV V)


% step 4b: assign proper sign to advective flux (3D matrix: destination boxes X source boxes X specific advection flux)
%          NOTE: AdvectionSign refers to whether a flux is a gain or a loss to a box, it does not refer to a onshore/offshore direction
AdvectionSign(1:5, 1:5, 1) = [
1	1	0	0	0
-1	-1	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % A_12 (matrix: spatial cell X spatial cell: I II III IV V)

AdvectionSign(1:5, 1:5, 2) = [
1	0	1	0	0
0	0	0	0	0
-1	0	-1	0	0
0	0	0	0	0
0	0	0	0	0]; % A_13 (matrix: spatial cell X spatial cell: I II III IV V)

AdvectionSign(1:5, 1:5, 3) = [
0	0	0	0	0
0	1	0	1	0
0	0	0	0	0
0	-1	0	-1	0
0	0	0	0	0]; % A_24 (matrix: spatial cell X spatial cell: I II III IV V)

AdvectionSign(1:5, 1:5, 4) = [
0	0	0	0	0
0	0	0	0	0
0	0	1	0	1
0	0	0	0	0
0	0	-1	0	-1]; % A_35 (matrix: spatial cell X spatial cell: I II III IV V)

AdvectionSign(1:5, 1:5, 5) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	1	0
0	0	0	0	0]; % A_4ocean (matrix: spatial cell X spatial cell: I II III IV V); (produces offshore loss to ocean)

AdvectionSign(1:5, 1:5, 6) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	1]; % A_5ocean (matrix: spatial cell X spatial cell: I II III IV V); (produces offshore loss to ocean)


% step 4c: define vertical mixing connections between boxes (3D matrix: DestinationBoxes X SourceSoxes X mixing flux)
VerticalMixingLink(1:5, 1:5, 1) = [
0	0	0	0	0
0	0	1	0	0
0	1	0	0	0
0	0	0	0	0
0	0	0	0	0]; % w_23

VerticalMixingLink(1:5, 1:5, 2) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	1
0	0	0	1	0]; % w_45


% step 4d: define horizontal mixing connections between boxes (3D matrix: DestinationBoxes X SourceSoxes X mixing flux)
HorizontalMixingLink(1:5, 1:5, 1) = [
0	1	0	0	0
1	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % u_12 (matrix: spatial cell X spatial cell: I II III IV V)

HorizontalMixingLink(1:5, 1:5, 2) = [
0	0	1	0	0
0	0	0	0	0
1	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % u_13 (matrix: spatial cell X spatial cell: I II III IV V)

HorizontalMixingLink(1:5, 1:5, 3) = [
0	0	0	0	0
0	0	0	1	0
0	0	0	0	0
0	1	0	0	0
0	0	0	0	0]; % u_24 (matrix: spatial cell X spatial cell: I II III IV V)

HorizontalMixingLink(1:5, 1:5, 4) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	1
0	0	0	0	0
0	0	1	0	0]; % u_35 (matrix: spatial cell X spatial cell: I II III IV V)

HorizontalMixingLink(1:5, 1:5, 5) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % u_4ocean (matrix: spatial cell X spatial cell: I II III IV V); NOTE: this does NOT define link between box 4 and ocean

HorizontalMixingLink(1:5, 1:5, 6) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % u_5ocean (matrix: spatial cell X spatial cell: I II III IV V); NOTE: this does NOT define link between box 5 and ocean


% step 4e: mixing face areas for horizontal mixing fluxes (u_12 u_13 u_24 u_35 u_4ocean u_5ocean)
BoxMixingFace = [BoxHeight_inshore(:, 2) BoxHeight_inshore(:, 3) BoxHeight_inshore(:, 4) BoxHeight_inshore(:, 5) BoxHeight_offshore(:, 4) BoxHeight_offshore(:, 5)] .* repmat(BoxWidth(:, 1), [1, num_HorizMixFluxes]); % (m2); (2D matrix: time X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean

% step 4f: define sinking connections between boxes (3D matrix: DestinationBoxes X SourceSoxes X sinking flux)
SinkLink(1:5, 1:5, 1) = [
-1	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0]; % S_1benthos

SinkLink(1:5, 1:5, 2) = [
0	0	0	0	0
0	-1	0	0	0
0	1	0	0	0
0	0	0	0	0
0	0	0	0	0]; % S_23

SinkLink(1:5, 1:5, 3) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	-1	0
0	0	0	1	0]; % S_45

SinkLink(1:5, 1:5, 4) = [
0	0	0	0	0
0	0	0	0	0
0	0	-1	0	0
0	0	0	0	0
0	0	0	0	0]; % S_3benthos

SinkLink(1:5, 1:5, 5) = [
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	0
0	0	0	0	-1]; % S_5benthos





% *************************************************************************
% STEP 5: net advection & mixing gains/losses to/from each box for each t--
AdvectionMatrix                  = zeros(num_t, num_boxes, num_boxes); % initialize; (3D matrix: time X source X destination (I II III IV V))
RiverFluxMatrix                  = zeros(num_t, num_boxes, num_boxes); % initialize; (3D matrix: time X source X destination (I II III IV V))
VerticalMixMatrix                = zeros(num_t, num_boxes, num_boxes); % initialize; (3D matrix: time X source X destination (I II III IV V))
HorizontalMixMatrix              = zeros(num_t, num_boxes, num_boxes); % initialize; (3D matrix: time X source X destination (I II III IV V))
HorizontalMix_OffshoreSurface    = zeros(num_t, 1);                    % initialize; (vertical vector: time X 1); u_4ocean
HorizontalMix_OffshoreSubSurface = zeros(num_t, 1);                    % initialize; (vertical vector: time X 1); u_5ocean
OceanSourceBoxID                 = zeros(num_t, 1);                    % initialize; (vertical vector)

for time_loop = 1:num_t % FFF can vectorize all this in future to speed things up
    
    current_BoxWidth                     = BoxWidth(time_loop, :);      % (m);  (horizontal vector: 1 X num_boxes)
    current_BoxWidth4flux                = repmat(BoxWidth(time_loop, 1), [1, num_HorizMixFluxes]);      % (m);  (horizontal vector: 1 X num_HorizMixFluxes)
    current_BoxLength                    = BoxLength(time_loop, :);     % (m);  (horizontal vector: 1 X num_boxes)
    current_BoxMixingFace                = BoxMixingFace(time_loop, :); % (m2); (horizontal vector: 1 X num_HorizMixFluxes)

    % step 5b: advection --------------------------------------------------
    current_advection                    = Advection_TimeSeries(time_loop, :);                                  %                           (horizontal vector: 1 X num_AdvecFluxes);                           (m3/s per 1m BoxWidth);
    current_advection                    = reshape(current_advection, 1, 1, num_AdvecFluxes);                   %                           (vector-along-layers: 1 X 1 X num_AdvecFluxes);                     (m3/s per 1m BoxWidth); 
    advection_repmat                     = repmat(current_advection, [num_boxes, num_boxes, 1]);                %                           (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);          (m3/s per 1m BoxWidth); 
    SwitchResult                         = AdvcDirectionSwitch .* advection_repmat;                             %                           (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);          (m3/s per 1m BoxWidth); value is negative where flow direction is wrong @ t
    looky_negative                       = find(SwitchResult <= 0);                                             % find zero or backwards flow entries in matrix
    advection_repmat(looky_negative)     = 0;                                                                   %                           (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);          (m3/s per 1m BoxWidth); set fluxes in source box clms to 0;
    advection_repmat                     = advection_repmat .* AdvectionSign;                                   %                           (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);          (m3/s per 1m BoxWidth); put correct sign on direction of flux (offshore is "+"); 
    AdvectionMatrix_temp                 = sum(advection_repmat, 3) * (60*60*24);                               % advection rate;           (2D matrix: DestinationBox X SourceBox);                            (m3/d per 1m BoxWidth)
                                                                                                                % sum all the individual advection fluxes into and out of each "destination" box,
    AdvectionMatrix_temp                 = AdvectionMatrix_temp  .* repmat(current_BoxWidth', [1, num_boxes]);  % scale by BoxWidth;        (2D matrix: DestinationBox X SourceBox);                            (m3/d); this is just a formality unless BoxWdith is not 1 m
    AdvectionMatrix(time_loop, :, :)     = reshape(AdvectionMatrix_temp', [1, num_boxes, num_boxes]);           %                           (3D matrix: time X SourceBox X DestinationBox);                     (m3/d); 
    
    % step 5c: identify upwelling vs downwelling events by source box ID --
    Advection_DestinationSum             = sum(AdvectionMatrix_temp, 2);                                        % advected volume (m3/d); (vertical vector: DestinationBoxes X 1)
    Advection_DestinationSum             = round2(Advection_DestinationSum, 8);                                 % round off the very tiny error caused by spline fit
    %   NOTE: all advection terms except those from external inputs (i.e. ocean or river) cancel out; 
    looky_ReceivingBox = find(Advection_DestinationSum < 0);
    if isempty(looky_ReceivingBox)
        OceanSourceBoxID(time_loop) = looky_OffshoreSurfaceBox;            % default to downwelling box if all advection = 0
    else
        OceanSourceBoxID(time_loop)  = find(Advection_DestinationSum < 0); % box number of box receiving advection from outside
    end
    
    % step 5d: riverine advection -----------------------------------------
    current_RiverFlux                    = RiverFlux_TimeSeries(time_loop, :);                                                              %                         (horizontal vector: 1 X num_AdvecFluxes);                             (m3/s per 1m BoxWidth); 
    current_RiverFlux                    = reshape(current_RiverFlux, 1, 1, num_AdvecFluxes);                                               %                         (vector-along-layers: 1 X 1 X num_AdvecFluxes)(m3/s per 1m BoxWidth); (m3/s per 1m BoxWidth);
    RiverFlux_repmat                     = repmat(current_RiverFlux, [num_boxes, num_boxes, 1]);                                            %                         (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);            (m3/s per 1m BoxWidth); 
    SwitchResult                         = AdvcDirectionSwitch .* RiverFlux_repmat;                                                         %                         (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);            (m3/s per 1m BoxWidth); value is negative where flow direction is wrong @ t
    looky_negative                       = find(SwitchResult <= 0);                                                                         % find zero or backwards flow entries in matrix
    RiverFlux_repmat(looky_negative)     = 0;                                                                                               %                         (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);            (m3/s per 1m BoxWidth); set fluxes in source box clms to 0; 
    RiverFlux_repmat                     = RiverFlux_repmat .* AdvectionSign;                                                               %                         (3D matrix: DestinationBox X SourceBox X num_AdvecFluxes);            (m3/s per 1m BoxWidth); put correct sign on direction of flux (offshore is "+");
    RiverFluxMatrix_temp                 = sum(RiverFlux_repmat, 3) * (60*60*24);                                                           % advection rate;         (2D matrix: DestinationBox X SourceBox);                              (m3/d per 1m BoxWidth); 
                                                                                                                                            % sum all the individual advection fluxes into and out of each "destination" box,
    RiverFluxMatrix_temp                 = RiverFluxMatrix_temp  .* repmat(current_BoxWidth', [1, num_boxes]);                              % scale by BoxWidth;      (2D matrix: DestinationBox X SourceBox);                              (m3/d); this is just a formality unless BoxWdith is not 1 m
    RiverFluxMatrix(time_loop, :, :)     = reshape(RiverFluxMatrix_temp', [1, num_boxes, num_boxes]);                                       % advection rate;         (3D matrix: time X SourceBox X DestinationBox);                       (m3/d)
    
    % step 5e: vertical mixing --------------------------------------------
    current_VertMixing                   = VerticalMixing_TimeSeries(time_loop, :);                                                         %                                      (horizontal vector: 1 X num_MixFluxes);                               (m/s);
    current_VertMixing                   = reshape(current_VertMixing, 1, 1, num_MixFluxes);                                                %                                      (vector-along-layers: 1 X 1 X num_MixFluxes);                         (m/s); 
    VertMixing_repmat                    = repmat(current_VertMixing, [num_boxes, num_boxes, 1]);                                           %                                      (3D matrix: DestinationBox X SourceBox X num_MixFluxes);              (m/s);
    VertMixing_repmat                    = VertMixing_repmat .* VerticalMixingLink;                                                         %                                      (3D matrix: DestinationBox X SourceBox X num_MixFluxes);              (m/s);
    VerticalMixMatrix_temp               = sum(VertMixing_repmat, 3) * (60*60*24);                                                          % vertical mixing rate;                (2D matrix: DestinationBox X SourceBox);                              (m/d);
    VerticalMixMatrix_temp               = VerticalMixMatrix_temp .* repmat((current_BoxLength .* current_BoxWidth)', [1, num_boxes]);      % scale by box size (horizontal area); (2D matrix: DestinationBox X SourceBox);                              (m3/d);
    VerticalMixMatrix(time_loop, :, :)   = reshape(VerticalMixMatrix_temp', [1, num_boxes, num_boxes]);                                     % mixing rate;                         (3D matrix: time X SourceBox X DestinationBox);                       (m3/d)
    
    % step 5f: horizontal mixing ------------------------------------------
    current_HorizMixing                         = HorizontalMixing_TimeSeries(time_loop, :);                % (m2/s); (horizontal vector: 1 X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    current_HorizMixing                         = current_HorizMixing * (60*60*24);                         % (m2/d); (horizontal vector: 1 X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    current_HorizMixing                         = current_HorizMixing .* current_BoxWidth4flux;             % (m3/d); NOTE: raw mix flux parameters already account for box height; (horizontal vector: 1 X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    current_HorizMixing                         = reshape(current_HorizMixing, 1, 1, num_HorizMixFluxes);   % (m3/d); (vector-along-layers: 1 X 1 X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    HorizMixing_repmat                          = repmat(current_HorizMixing, [num_boxes, num_boxes, 1]);   % (m3/d); (3D matrix: DestinationBox X SourceBox X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    HorizMixing_repmat                          = HorizMixing_repmat .* HorizontalMixingLink;               % (m3/d); (3D matrix: DestinationBox X SourceBox X num_HorizMixFluxes); u_12 u_13 u_24 u_35 u_4ocean u_5ocean
    HorizMixing_repmat                          = sum(HorizMixing_repmat, 3);                               % (m3/d); (2D matrix: DestinationBox X SourceBox);
    HorizontalMixMatrix(time_loop, :, :)        = reshape(HorizMixing_repmat', [1, num_boxes, num_boxes]);  % horizontal mixing rate; (m3/d); (3D matrix: time X SourceBox X DestinationBox);
    HorizontalMix_OffshoreSurface(time_loop)    = current_HorizMixing(1, 1, 5);                             % horizontal mixing rate u_4ocean; (m3/d); (vertical vector: time X 1)
    HorizontalMix_OffshoreSubSurface(time_loop) = current_HorizMixing(1, 1, 6);                             % horizontal mixing rate u_5ocean; (m3/d); (vertical vector: time X 1)
        
end

% step 5g: add advection terms (wind-drive advection + river-driven advection)
NetAdvectionMatrix = AdvectionMatrix + RiverFluxMatrix; % net advection rate (m3/d); (3D matrix: time X SourceBox X DestinationBox (I II III IV V))





% *************************************************************************
% STEP 6: sinking fluxes to/from each box (identical for each t; t dimension not defined)
SinkLink                                = sum(SinkLink, 3);                % flatten SinkLink; (2D matrix: destination boxes X source boxes)
SinkLink_surface                        = zeros(num_boxes);
SinkLink_surface(:, looky_SurfaceBoxes) = SinkLink(:, looky_SurfaceBoxes); % only consider sinking from surface to sub-surface boxes
SinkLink_benthos                        = SinkLink;
SinkLink_benthos(:, looky_SurfaceBoxes) = 0;                               % only consider sinking from sub-surface to benthos (also includes surface to benthos mixed-boxes (Box I))





% *************************************************************************
% STEP 7: load nutrient data-----------------------------------------------
%         The file cgoa_ancyc.mat contains two variables:
% 
%           ancyc_cgoa_ctd       1x5              8832  struct              
%           ancyc_cgoa_nuts      1x5              4416  struct              
% 
%         The five fields refer to the model boxes 1 through 5, and contains
%         the following fields (e.g., for ancyc_cgoa_nuts):
% 
%           ancyc_cgoa_nuts = 
%               1x5 struct array with fields:
%                   mean_no3no2        12 monthly means
%                   num_no3no2         number of points used to compute the mean
%                   std_no3no2         standard deviation
%                   intp_no3no2        12 monthly means with missing months interpolated
%     
%         The structure ancyc_cgoa_ctd contains respective fields for 
%         the variables temperature ("t") and salinity ("s"):
% 
%           ancyc_cgoa_ctd = 
%               1x5 struct array with fields:
%                   mean_t
%                   mean_s
%                   num_t
%                   num_s
%                   std_t
%                   std_s
%                   intp_t
%                   intp_s
%
%         FFF could add river nutrient input in future
NutrientFile_directory     = '/Users/ruzickaj/Documents/ECOTRAN_08052018/ECOTRAN_code/Physics_code/';
readNutrientFile           = 'cgoa_ancyc.mat'; % monthly mean nutrients
readNutrientFile           = [NutrientFile_directory readNutrientFile];
load(readNutrientFile, 'ancyc_cgoa_nuts')
box_I_NO3                  = ancyc_cgoa_nuts(1).intp_no3no2;
box_II_NO3                 = ancyc_cgoa_nuts(2).intp_no3no2;
box_III_NO3                = ancyc_cgoa_nuts(3).intp_no3no2;
box_IV_NO3                 = ancyc_cgoa_nuts(4).intp_no3no2;
box_V_NO3                  = ancyc_cgoa_nuts(5).intp_no3no2;
dat_NO3                    = [box_I_NO3' box_II_NO3' box_III_NO3' box_IV_NO3' box_V_NO3']; % monthly mean NO3 + NO2; (micro-moles N/L) = (mmoles N/m3); (matrix: month X box; I, II, III, IV, V)

% step 7b: interpolate monthly means to daily values ----------------------
% allow for cyclical repeats over several years
build_NO3           = [];
for year_loop = 1:num_years
    build_NO3 = [build_NO3; dat_NO3];
end
NO3_doy       = interp1([1 build_midmonth_doy (365*num_years)]', [mean(dat_NO3([1 12], :)); build_NO3; mean(dat_NO3([1 12], :))], t_grid, 'spline'); % (2D matrix: num_t X num_AdvecFluxes); (m/s)
NO3_doy(NO3_doy < 0) = 0; % filter out negative NO3_doy values created during spline interpolation





% *************************************************************************
% STEP 8: WWT conversion terms---------------------------------------------
atomic_mass_N             = 14.00674;   % (g N / mole N)
atomic_mass_C             = 12.0107;    % (g C / mole C)

Chla_to_N_diatoms         = 2.19;       % (mg Chla mmoles N^-1) (Dickson & Wheeler 1995 L&O 40(3):533-543)
% Chla_to_N_diatoms         = 2.5;        % 2.49 to 2.67 (mg Chla mmoles N^-1) (Chan 1980 J Phycol 16:428-432)
Chla_to_N_dinoflagellates = 0.84;       % 0.73 to 0.95 (mg Chla mmoles N^-1) (Chan 1980 J Phycol 16:428-432)

% C_to_N_phytoplankton      = 7.3;        % general phytoplankton C:N (moles C / moles N) (Geider & LaRoche 2002 Eur. J. Phycol 37:1-17)
C_to_N_phytoplankton      = (106/16);   % Redfield ratio = C:N:P = 106:16:1 ---> C:N = 106:16 = 6.625 mole C/mole N
C_to_N_zooplankton        = 5;          % general zooplankton C:N (moles C moles N^-1) for mesozooplankton (W. Peterson pers com); C:N=5.16-5.26 [Schneider 1990 (Mar Biol 106:219-225)]; C:N=3.9 [Uye 1982]

WWT_to_C                  = 8.77;       % (g WWT / g C); (Steele et al. 2007), NOTE: I could not find this value in 2007 citation, must have gotten value directly from John, value is close to other lit. values; {Strickland, 1966 #1521} says "From the foregoing, it appears that the 10-12 mg algal weight:mg C is the most suitable factor for estimating phytoplankton biomass as ?wet weight?";
mgWWT_per_mmolesN         = (C_to_N_phytoplankton * atomic_mass_C * WWT_to_C); % (mg WWT/mmoles N)

mgram_to_gram             = 1000;       % (mg/g)
gram_to_ton               = 1000000;	% (g/t)
mgram_to_ton              = mgram_to_gram * gram_to_ton; % (mg/t)

W_to_microE               = 4.6;        % convert (W m^-2) to (microE m^-2 s^-1) at 400-700nm (Brock 1981 Ecol Model 14: 1-19)





% *************************************************************************
% STEP 9: light parameters-------------------------------------------------
LightParams.latitude        = latitude;   % latitude of NH line (decimal degress north) based on LTOP transects
LightParams.SolarConstant	= 1366;       % Solar const. (W m^-2) recent ave. (Froehlich & Lean (1998, Geographical Research Letters 25(23): 4377-4380))

LightParams.par_frac        = 0.43;       % fraction of 40 that is PAR, clear atmos. (Fasham, Ducklow, & McKelvie (1990, J. Mar. Res. 48:591-639), p. 604)
LightParams.surf_trans      = 0.85;       % 1-reflectance (Parsons, Takahachi, & Hargrave (1984, Biological Oceanographic Processes, 3rd ed.), p. 69)
LightParams.cloud_cover     = 0.50;       % fitting daily average and weekly-smoothed light model curve to weekly average of HMSC roof data by eye
% LightParams.cloud_cover     = 0.70;       % Ave. 1990-97, COADS Standard 1-deg. for 125.5W, 45.5N
LightParams.cloud_trans     = 1.0 - (0.7 * LightParams.cloud_cover);  % (Evans and Parslow (1985, Biol. Oceanogr. 3(3):327-347), p. 344)
LightParams.t_grid          = t_grid;

[SurfaceRadiation]          = f_LightIntensity(LightParams);
daily_average_light         = SurfaceRadiation.daily_average_light; % daily integrated solar raditation at ocean surface averaged across 24 hours (W m^-2 h^-1); (vertical vector: num_t X 1)
%                                                 NOTE: the daily average is also what was used by Spitz in her NPZD models
%                                                 NOTE: the daily integrated value avergaed over 24 hours is almost identical to the mean of the current instantaneous light intensity averaged over 24 hours (I checked)
current_light               = SurfaceRadiation.current_light; % surface solar raditation at current time (W/m2)
Kw                          = 0.067;    % Light attenuation_seawater (Newberger et al., 2003); (1/m)
Kp                          = 0.0095;   % Light attenuation_phytoplankton (Newberger et al., 2003); (m2/mmol N)





% *************************************************************************
% STEP 10: pack results for export-----------------------------------------
% step 10a: time dimensions -----------------------------------------------
ECOTRANphysics.datestart                   = datestart;                 % starting date
ECOTRANphysics.dateend                     = dateend;                   % ending date
ECOTRANphysics.t_grid                      = t_grid;
ECOTRANphysics.num_t                       = num_t;

% step 10b: space dimensions ----------------------------------------------
ECOTRANphysics.num_boxes                   = num_boxes;
ECOTRANphysics.BoxLength                   = BoxLength;                 % (m); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxHeight_inshore           = BoxHeight_inshore;         % I, II, III, IV, V; (m); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxHeight_offshore          = BoxHeight_offshore;        % I, II, III, IV, V; (m); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxHeight                   = BoxHeight;                 % I, II, III, IV, V; mean box height; (m); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxWidth                    = BoxWidth;                  % I, II, III, IV, V; (m); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxArea                     = BoxArea;                   % I, II, III, IV, V; (m2); (2D matrix: time X num_boxes)
ECOTRANphysics.BoxVolume                   = BoxVolume;                 % I, II, III, IV, V; (m3); (2D matrix: time X num_boxes)

% step 10c: box addresses -------------------------------------------------
ECOTRANphysics.looky_SubSurfaceBoxes       = looky_SubSurfaceBoxes;
ECOTRANphysics.looky_SurfaceBoxes          = looky_SurfaceBoxes;
ECOTRANphysics.looky_OffshoreSurfaceBox    = looky_OffshoreSurfaceBox;
ECOTRANphysics.looky_OffshoreSubSurfaceBox = looky_OffshoreSubSurfaceBox;
ECOTRANphysics.looky_RiverFluxBox          = looky_RiverFluxBox;        % Optional river alongshore input; Box 2 in CGoA model
ECOTRANphysics.OceanSourceBoxID            = OceanSourceBoxID;          % ocean sourcebox ID; (vertical vector: time X 1)

% step 10d: raw physical driver time-series -------------------------------
ECOTRANphysics.tau                         = tau;                       % Alongshore wind stress, monthly mean: (N/m^2) = (kg*m/s2/m2) = (kg/s2/m); (horizontal vector)
ECOTRANphysics.tau_doy                     = tau_doy;                   % Alongshore wind stress, monthly mean: (N/m^2) = (kg*m/s2/m2) = (kg/s2/m); (horizontal vector)
ECOTRANphysics.we                          = we;                        % vertical mixing rate, monthly mean: (m/sec)
ECOTRANphysics.we_doy                      = we_doy;                    % vertical mixing rate, monthly mean: (m/sec)
ECOTRANphysics.ue                          = ue;                        % horizontal mixing rate, monthly mean: (m/s); (2d matrix: time X num_AdvecFluxes)
ECOTRANphysics.ue_doy                      = ue_doy;                    % horizontal mixing rate, monthly mean: (m2/s); (2d matrix: time X num_AdvecFluxes)

% step 10e: physical driver time-series -----------------------------------
ECOTRANphysics.AdvectionMatrix             = AdvectionMatrix;           % wind-driven advection rate = volume transported per day; (m3/d); (3D matrix: time X source box X destination box)
ECOTRANphysics.RiverFluxMatrix             = RiverFluxMatrix;           % river-driven advection rate = volume transported per day; enters into Box II; (m3/d); (3D matrix: time X source box X destination box)
ECOTRANphysics.VerticalMixMatrix           = VerticalMixMatrix;         % mixing rate = volume mixed per day; (m3/d); (3D matrix: time X source box X destination box)
ECOTRANphysics.HorizontalMixMatrix         = HorizontalMixMatrix;       % mixing rate = volume mixed per day; (m3/d); (3D matrix: time X source box X destination box)

ECOTRANphysics.HorizontalMix_OffshoreSurface    = HorizontalMix_OffshoreSurface;    % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)
ECOTRANphysics.HorizontalMix_OffshoreSubSurface = HorizontalMix_OffshoreSubSurface; % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)

ECOTRANphysics.NetAdvectionMatrix          = NetAdvectionMatrix;        % net advection rate (m3/d); (3D matrix:time X SourceBox X DestinationBox (I II III IV V))
ECOTRANphysics.BoxWashoutTime              = looky_WashoutTime;         % a list of days when one or more boxes would have been >100% washed out if advection cap not applied

% step 10f: misc info -----------------------------------------------------
ECOTRANphysics.SinkLink_surface            = SinkLink_surface;          % define sinking connections from surface to sub-surface boxes; (unitless); (2D matrix: destination boxes X source boxes)
ECOTRANphysics.SinkLink_benthos            = SinkLink_benthos;          % define sinking connections from sub-surface boxes to benthic detritus; (unitless); (2D matrix: destination boxes X source boxes)
ECOTRANphysics.num_AdvecFluxes             = num_AdvecFluxes;
ECOTRANphysics.num_MixFluxes               = num_MixFluxes;
ECOTRANphysics.num_HorizMixFluxes          = num_HorizMixFluxes;

% step 10g: nutrient time-series ------------------------------------------
ECOTRANphysics.dat_NO3                     = dat_NO3;                   % monthly mean NO3 + NO2; (mmole N/m3); (2D matrix: month X box; I, II, III, IV, V)
ECOTRANphysics.NO3_doy                     = NO3_doy;                   % daily NO3 + NO2 interpolated from monthly means; (mmole N/m3); (2D matrix: month X box; I, II, III, IV, V)

% step 10h: light model info ----------------------------------------------
ECOTRANphysics.MLD                         = MLD;                       % time-series of Mixed Layer Depth (for light intensity calculations, NOT advection calcs); (m); (vertical vector: length = time)
ECOTRANphysics.Io                          = daily_average_light;       % time-series of surface PAR light intensity; daily integrated solar raditation at ocean surface averaged across 24 hours; (W m^-2 h^-1) (Brock, 1981, R. Ozretich unpub. data);
ECOTRANphysics.Kw                          = Kw;                        % Light attenuation?seawater
ECOTRANphysics.Kp                          = Kp;                        % Light attenuation?phytoplankton (Newberger et al., 2003); (m2/mmol N)

% step 10i: conversion factors --------------------------------------------
ECOTRANphysics.atomic_mass_C               = atomic_mass_C;             % (g C / mole C)
ECOTRANphysics.atomic_mass_N               = atomic_mass_N;             % (g N / mole N)
ECOTRANphysics.C_to_N_phytoplankton        = C_to_N_phytoplankton;      % Redfield ratio = C:N:P = 106:16:1 ---> C:N = 106:16 = 6.625 mole C/mole N
ECOTRANphysics.C_to_N_zooplankton          = C_to_N_zooplankton;        % general zooplankton C:N (moles C moles N^-1) for mesozooplankton (W. Peterson pers com); C:N=5.16-5.26 [Schneider 1990 (Mar Biol 106:219-225)]; C:N=3.9 [Uye 1982]
ECOTRANphysics.Chla_to_N_diatoms           = Chla_to_N_diatoms;         % (mg Chla mmoles N^-1) (Dickson & Wheeler 1995 L&O 40(3):533-543)
ECOTRANphysics.Chla_to_N_dinoflagellates   = Chla_to_N_dinoflagellates;
ECOTRANphysics.mgWWT_per_mmolesN           = mgWWT_per_mmolesN;         % (mg WWT/mmoles N); for phytoplankton
ECOTRANphysics.WWT_to_C                    = WWT_to_C;                  % (g WWT / g C); (Steele et al. 2007); QQQ recheck this for phytoplankton


% end m-file***************************************************************