function dy = f_ECOTRANode_ReflectiveBoundary_05182017(t, ProductionRates_t, ODEinput)
% generic ODE for ECOTRANdynamic with 5-box models
% by Jim Ruzicka
% takes:
%   ODEinput
%       (trophic model info)
%           EnergyBudget_matrix              consumption fates                                  (fraction)      (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
%           EnergyBudget_BoxType             turn off specific trophic linkages in sub-surface boxes            (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
%           BioenergeticBudget_feces         consumption fraction to feces                                      (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_metabolism    consumption fraction to metabolism                                 (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_predation     consumption fraction to predation                                  (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_eggs          consumption fraction to eggs                                       (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_Senescence    consumption fraction to Senescence                                 (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_BA            consumption fraction to BA                                         (3D matrix: num_t X num_E2Egroups X num_boxes)
%           BioenergeticBudget_EM            consumption fraction to EM                                         (3D matrix: num_t X num_E2Egroups X num_boxes)
%           DetritusFate_feces                                                                                  (3D matrix: 2 X num_E2Egroups X num_boxes)
%           DetritusFate_senescence                                                                             (3D matrix: 2 X num_E2Egroups X num_boxes)
%           ExcretionFate                                                                                       (3D matrix: 2 X num_E2Egroups X num_boxes)
%           EggFate                                                                                             (3D matrix: num_eggs X num_E2Egroups X num_boxes)
%           SpecificGrowthRate              intrinsic (weight-specific) growth rates (not used) (1/d)           (3D matrix: num_t X num_E2Egroups X num_boxes)
%           SpecificConsumptionRate         intrinsic (weight-specific) consumption rates       (1/d)           (3D matrix: num_t X num_E2Egroups X num_boxes)
%           TransferEfficiency                                                                  (fraction)      (3D matrix: 1 X num_E2Egroups X num_boxes)
%       (production rates: initial conditions, external inputs)
%           Production_input                daily advection rates of NO3 INTO model domain      (mmole N/m3/d)  (3D matrix: num_t X num_E2Egroups X num_boxes)
%           Production_initial              initial & mean production rates (consumption)       (mmole N/m3/d)  (3D matrix of vertical vectors: num_E2Egroups X 1 X num_boxes)
%           ProductionC_initial_repmat      mean production rates for functional response calcs (mmole N/m3/d)  (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes); NOTE: vertical vectors
%           BoxBiomass_OceanSurface         ocean NO3 conc. for horizontal mixing calcs         (mmole N/m3)    (vertical vector: num_E2Egroups X 1); NOTE: could define ALL group ocean biomasses here
%           BoxBiomass_OceanSubSurface      ocean NO3 conc. for horizontal mixing calcs         (mmole N/m3)    (vertical vector: num_E2Egroups X 1); NOTE: could define ALL group ocean biomasses here
%       (addresses and numbers in EnergyBudget_matrix)
%           looky_NO3
%           looky_plgcNH4
%           looky_bnthNH4
%           looky_nonNO3
%           looky_DETRITUS
%           looky_terminalPlgcDetr
%           looky_terminalBnthDetr
%           looky_ANYConsumer
%           looky_fishery
%           looky_eggs
%           num_eggs
%           num_bnthNH4
%           E2E_rows                        number of ECOTRAN consumers (= num_E2Egroups)
%           E2E_clms                        number of ECOTRAN producers (= num_E2Egroups)
%       (physical model info)
%       	AdvectionMatrix                 net advection rate                                  (m3/d)          (3D matrix: time X SourceBox X DestinationBox (I II III IV V))
%       	RiverFluxMatrix                 net advection rate                                  (m3/d)          (3D matrix: time X SourceBox X DestinationBox (I II III IV V))
%       	VerticalMixMatrix               vertical mixing                                     (m3/d)          (3D matrix: groups X SourceBox X DestinationBox)
%       	HorizontalMixMatrix             horizontal mixing                                   (m3/d)          (3D matrix: groups X SourceBox X DestinationBox)
%           SinkFrac                        fraction of total production sinking OUT of source box              (3D matrix: num_t X num_E2Egroups X num_boxes)
%           t_grid                          day number                                                          (vertical vector: time x 1)
%       (addresses and numbers in physical model)
%           looky_SurfaceBoxes
%           looky_SubSurfaceBoxes
%           looky_OffshoreSurfaceBox
%           looky_OffshoreSubSurfaceBox
%           num_boxes                       number of spatial boxes in model
%       (functional response parameters)
%           FunctionalResponseParams        producer vulnerability (m_p) (3D matrix: group X PRODUCERS X num_boxes), replicated down rows
%                   m_p = 0 is "linear"
%                   m_p = 1 is "non-linear"  ECOSIM default; half-sat constant of individual groups 
%       (used only for Michaelis-Menten purposes)
%           looky_PrimaryProducer
%           looky_Macroalgae
%           num_PrimaryProducers
%           num_Macroalgae
%           Kw                              light attenuation_seawater                                          (scaler)
%           Kp                              ight attenuation_phytoplankton                      (m2/mmol N)     (scaler)
%           MichaelisMenten_Vmax            Vmax  = maximum nutrient uptake rate                (1/d)           (3D matrix: num_PrimaryProducers X 1 X num_boxes)
%           MichaelisMenten_KNO3            KNO3  = NO3 half-saturation constant                (mmol N/m3)     (3D matrix: num_PrimaryProducers X 1 X num_boxes)
%           MichaelisMenten_KNH4            KNH4  = NH4 half-saturation constant                (mmol N/m3)     (3D matrix: num_PrimaryProducers X 1 X num_boxes)
%           MichaelisMenten_alpha           alpha = initial slope of light response curve       (m2/W/d)        (3D matrix: num_PrimaryProducers X 1 X num_boxes)
%           MichaelisMenten_psi             psi   = NO3 uptake inhibition by NH4                (m3/mmole N)    (3D matrix: num_PrimaryProducers X 1 X num_boxes)
%   time-step & current production rate supplied by ode45
%           t                               current time-step
%       ProductionRates_t                   (AKA "y") current production rates of each group    (t WWT/km2*m/d) (vertical vectors: groups X 1 X num_boxes)
% returns:
%           dy                              daily rate of change of production rate              (t/km2/d2)     (vertical vector)
% calls:
%       f_MichaelisMenten_05152016
%
% NOTE: ProductionEfficiency .* Consumption_INflow converts Consumption_INflow from
%       consumption to predation and accounts for metabolism & non-assimilation as linear terms
% NOTE: TransferEfficiency .* Consumption_INflow converts Consumption_INflow from
%       consumption to predation and accounts for metabolism &
%       non-assimilation as linear terms. 
% NOTE: In ECOTRAN, Senescence, Non-Assimilation, & metabolism are
%       accounted for in the EnergyBudget_matrix, and are therefore
%       mathematically equivalent to functional num_E2Egroups, and flow to
%       Senescence, Non-Assimilation, & metabolism is handled the same way as
%       flow to predators.
% NOTE: Reflective ocean boundary conditions. (Just a cleaned-up version of f_ECOTRANode_09302016.m)
 % revision date: 5-18-2016




% *************************************************************************
% STEP 1: unpack parameters & drivers from ODEinput------------------------
% step 1a: EnergyBudget_matrix --------------------------------------------
EnergyBudget_matrix               = ODEinput.EnergyBudget_matrix;               % (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
EnergyBudget_BoxType              = ODEinput.EnergyBudget_BoxType;              % (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)

% step 1b: BioenergeticBudget terms ---------------------------------------
BioenergeticBudget_feces          = ODEinput.BioenergeticBudget_feces;          % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_metabolism     = ODEinput.BioenergeticBudget_metabolism;     % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_predation      = ODEinput.BioenergeticBudget_predation;      % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_eggs           = ODEinput.BioenergeticBudget_eggs;           % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_Senescence     = ODEinput.BioenergeticBudget_OtherMortality; % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_BA             = ODEinput.BioenergeticBudget_BA;             % (3D matrix: num_t X num_E2Egroups X num_boxes)
BioenergeticBudget_EM             = ODEinput.BioenergeticBudget_EM;             % (3D matrix: num_t X num_E2Egroups X num_boxes)

% step 1c: addresses & counts ---------------------------------------------
looky_NO3                         = ODEinput.looky_NO3;
looky_plgcNH4                     = ODEinput.looky_plgcNH4;
looky_bnthNH4                     = ODEinput.looky_bnthNH4;
looky_nonNO3                      = ODEinput.looky_nonNO3;
looky_DETRITUS                    = ODEinput.looky_DETRITUS;
looky_terminalPlgcDetr            = ODEinput.looky_terminalPlgcDetr;
looky_terminalBnthDetr            = ODEinput.looky_terminalBnthDetr;
looky_PrimaryProducer             = ODEinput.looky_PrimaryProducer;
looky_Macroalgae                  = ODEinput.looky_Macroalgae;
looky_ANYConsumer                 = ODEinput.looky_ANYConsumer;
looky_fishery                     = ODEinput.looky_fishery;
looky_eggs                        = ODEinput.looky_eggs;
looky_SurfaceBoxes                = ODEinput.looky_SurfaceBoxes;
looky_SubSurfaceBoxes             = ODEinput.looky_SubSurfaceBoxes;
looky_OffshoreSurfaceBox          = ODEinput.looky_OffshoreSurfaceBox;
looky_OffshoreSubSurfaceBox       = ODEinput.looky_OffshoreSubSurfaceBox;
num_boxes                         = ODEinput.num_boxes;             % number of spatial boxes in model
num_eggs                          = ODEinput.num_eggs;
num_bnthNH4                       = ODEinput.num_bnthNH4;
% num_PrimaryProducers              = ODEinput.num_PrimaryProducers;
num_Macroalgae                    = ODEinput.num_Macroalgae;
E2E_rows                          = ODEinput.E2E_rows;
E2E_clms                          = ODEinput.E2E_clms;

% step 1d: parameters -----------------------------------------------------
DetritusFate_feces                = ODEinput.DetritusFate_feces;                % (3D matrix: 2 X num_E2Egroups X num_boxes)
DetritusFate_senescence           = ODEinput.DetritusFate_senescence;           % (3D matrix: 2 X num_E2Egroups X num_boxes)
ExcretionFate                     = ODEinput.ExcretionFate;                     % (3D matrix: 2 X num_E2Egroups X num_boxes)
EggFate                           = ODEinput.EggFate;                           % (3D matrix: num_eggs X num_E2Egroups X num_boxes)
% SpecificGrowthRate                = ODEinput.SpecificGrowthRate;                % (1/d); (3D matrix: num_t X num_E2Egroups X num_boxes); NOTE: use SpecificConsumptionRate instead
SpecificConsumptionRate           = ODEinput.SpecificConsumptionRate;           % (1/d); (3D matrix: num_t X num_E2Egroups X num_boxes)
TransferEfficiency                = ODEinput.TransferEfficiency;                % (3D matrix: 1 X num_E2Egroups X num_boxes)
t_grid                            = ODEinput.t_grid;                            % (vertical vector: time x 1)

FunctionalResponseParams          = ODEinput.FunctionalResponseParams; % producer vulnerability (m_p); (3D matrix: group X PRODUCERS X num_boxes), replicated down rows
MichaelisMenten_Vmax              = ODEinput.MichaelisMenten_Vmax;  % Vmax  = maximum nutrient uptake rate;          (1/d);        (3D matrix: num_PrimaryProducers X 1 X num_boxes)
MichaelisMenten_KNO3              = ODEinput.MichaelisMenten_KNO3;  % KNO3  = NO3 half-saturation constant;          (mmol N/m3);  (3D matrix: num_PrimaryProducers X 1 X num_boxes)
MichaelisMenten_KNH4              = ODEinput.MichaelisMenten_KNH4;  % KNH4  = NH4 half-saturation constant;          (mmol N/m3);  (3D matrix: num_PrimaryProducers X 1 X num_boxes)
MichaelisMenten_alpha             = ODEinput.MichaelisMenten_alpha; % alpha = initial slope of light response curve; (m2/W/d);     (3D matrix: num_PrimaryProducers X 1 X num_boxes)
MichaelisMenten_psi               = ODEinput.MichaelisMenten_psi;   % psi   = NO3 uptake inhibition by NH4;          (m3/mmole N); (3D matrix: num_PrimaryProducers X 1 X num_boxes)

ProductionFraction_macroalgae     = ODEinput.ProductionFraction_macroalgae; % tie any macroalgae production to a fixed multiple of total phytoplankton production in each box; (3D matrix: num_Macroalgae X 1 X num_boxes)                                                            % consumption inflow rate;           (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes) 


% step 1d: Physics terms --------------------------------------------------
SinkFrac                          = ODEinput.SinkFrac;              % fraction of total production sinking OUT of source box; (unitless); (3D matrix: num_t X num_E2Egroups X num_boxes)
AdvectionMatrix                   = ODEinput.AdvectionMatrix;       % volume transported per day; (m3/d); (3D matrix: time X SourceBox X DestinationBox (I II III IV V))
RiverFluxMatrix                   = ODEinput.RiverFluxMatrix;       % volume transported per day; (m3/d); (3D matrix: time X SourceBox X DestinationBox (I II III IV V))
VerticalMixMatrix                 = ODEinput.VerticalMixMatrix;     % vertical mixing rate        (m3/d); (3D matrix: time X SourceBox X DestinationBox)
HorizontalMixMatrix               = ODEinput.HorizontalMixMatrix;   % horizontal mixing rate      (m3/d); (3D matrix: time X SourceBox X DestinationBox)
HorizontalMix_OffshoreSurface     = ODEinput.HorizontalMix_OffshoreSurface;    % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)
HorizontalMix_OffshoreSubSurface  = ODEinput.HorizontalMix_OffshoreSubSurface; % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)
SinkLink_surface                  = ODEinput.SinkLink_surface;      % define sinking connections from surface to sub-surface boxes; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes)
SinkLink_benthos                  = ODEinput.SinkLink_benthos;      % define sinking connections from sub-surface boxes to benthic detritus; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes)
RetentionScaler                   = ODEinput.RetentionScaler;       % has spatial structure (1 X num_E2Egroups X num_boxes)
% BoxHeight                         = ODEinput.BoxHeight;             % (m); (2D matrix: time X num_boxes)
BoxVolume                         = ODEinput.BoxVolume;             % (m3); (2D matrix: time X num_boxes)
MLD                               = ODEinput.MLD;                   % time-series of Mixed Layer Depth (for light intensity calculations, NOT advection calcs); (m); (vertical vector: length = num_t)
Io                                = ODEinput.Io;                    % time-series of surface PAR light intensity; daily integrated solar raditation at ocean surface averaged across 24 hours; (W m^-2 h^-1); NOTE: the daily average is also what was used by Spitz in her NPZD models;
% Kw                                = ODEinput.Kw;                    % Light attenuation of seawater; (scaler)
% Kp                                = ODEinput.Kp;                    % Light attenuation of phytoplankton (Newberger et al., 2003); (m2/mmol N)

% step 1e: Production driver, initial Production (actually initial consumption inflow), & dynamic input Production @ t
Production_input                  = ODEinput.Production_input;              % production added to each box via physics per day (mmole N/m3/d); (3D matrix: time X num_E2Egroups X num_boxes)
Production_initial                = ODEinput.Production_initial;            % production rates to use as "initial" conditions; (mmole N/m3/d); (3D matrix of vertical vectors: num_E2Egroups X 1 X num_boxes)
ProductionC_initial_repmat        = ODEinput.ProductionC_initial_repmat;    % consumption inflow rate; (mmole N/m3/d); (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes); NOTE: vertical vectors
BoxBiomass_OceanSurface           = ODEinput.BoxBiomass_OceanSurface;       % ocean nutrient concentrations; for horizontal mixing calcs; (vertical vector: num_E2Egroups X 1); (mmole N/m3)
BoxBiomass_OceanSubSurface        = ODEinput.BoxBiomass_OceanSubSurface;    % ocean nutrient concentrations; for horizontal mixing calcs; (vertical vector: num_E2Egroups X 1); (mmole N/m3)

ProductionRates_t                 = reshape(ProductionRates_t, [E2E_clms, 1, num_boxes]); % "unstack" vertical vector of ProductionRates_t (3D matrix: num_E2Egroups X 1 X num_boxes); (mmole N/m3/d)
% % QQQ for de-bugging
% t=75;
% ProductionRates_t = Production_initial;

% step 1f: filter out sub-minimum (negative) Production @ t ---------------
min_ProductionRates                                        = 0;
ProductionRates_t(ProductionRates_t < min_ProductionRates) = min_ProductionRates; % (mmole N/m3/d); (3D matrix: num_E2Egroups X 1 X num_boxes)

% step 1g: initialize dy --------------------------------------------------
dy                                = zeros(1, E2E_clms, num_boxes); % initialize output; will get transposed to vertical vector at end

% % step 1h: scenario rules -------------------------------------------------
% %          NOTE: this could be anything, using a vertical vector for scaling NO3 levels for now
% Scenario_scaler                    = ODEinput.Scenario_scaler;     % scenario time-series (scaler); (3D matrix: time X num_E2Egroups X num_boxes)





% *************************************************************************
% STEP 2: interpolate input production & parameters @ t--------------------
% step 2a: BioenergeticBudget terms @ t -----------------------------------
%          (allow for defined seasonal changes in BioenergeticBudget terms)
%          (as of 5/6/2015, BioenergeticBudget terms are defined as constant through time)
QBudget_feces_t           = interp1(t_grid, BioenergeticBudget_feces,       t); % BioenergeticBudget_feces @ t;      (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_metabolism_t      = interp1(t_grid, BioenergeticBudget_metabolism,  t); % BioenergeticBudget_metabolism @ t; (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_predation_t       = interp1(t_grid, BioenergeticBudget_predation,   t); % BioenergeticBudget_predation @ t;  (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_eggs_t            = interp1(t_grid, BioenergeticBudget_eggs,        t); % BioenergeticBudget_eggs @ t;       (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_Senescence_t      = interp1(t_grid, BioenergeticBudget_Senescence,  t); % BioenergeticBudget_Senescence @ t; (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_BA_t              = interp1(t_grid, BioenergeticBudget_BA,          t); % BioenergeticBudget_BA @ t;         (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)
QBudget_EM_t              = interp1(t_grid, BioenergeticBudget_EM,          t); % BioenergeticBudget_EM @ t;         (fraction); (3D matrix: 1 X num_E2Egroups X num_boxes)

% step 2b: weight-specific growth and consumption rates @ t ---------------
% SpecificGrowthRate_t      = interp1(t_grid, SpecificGrowthRate,             t); % (1/d); (3D matrix: 1 X num_E2Egroups X num_boxes); NOTE: use SpecificConsumptionRate instead
SpecificConsumptionRate_t = interp1(t_grid, SpecificConsumptionRate,        t); % (1/d); (3D matrix: 1 X num_E2Egroups X num_boxes)

% step 2c: driver production @ t ------------------------------------------
ProductionInput_t         = interp1(t_grid, Production_input,               t); % Production_input @ t; external input to model domain; (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X num_boxes)

% step 2d: box dimensions @ t ---------------------------------------------
% BoxHeight_t               = interp1(t_grid, BoxHeight,                      t); % (m);  (horizontal vector: 1 X num_boxes)
BoxVolume_t               = interp1(t_grid, BoxVolume,                      t); % (m3); (horizontal vector: 1 X num_boxes)

% step 2e: physical advection & mixing @ t --------------------------------
Advection_t               = interp1(t_grid, AdvectionMatrix,                t); % Advection @ t         (m3/d); (3D matrix: 1 X SourceBox X DestinationBox); positive & negative values
RiverFlux_t               = interp1(t_grid, RiverFluxMatrix,                t); % Advection @ t         (m3/d); (3D matrix: 1 X SourceBox X DestinationBox); positive & negative values
VerticalMix_t             = interp1(t_grid, VerticalMixMatrix,              t); % Vertical Mixing @ t   (m3/d); (3D matrix: 1 X SourceBox X DestinationBox); all positive values
HorizontalMix_t           = interp1(t_grid, HorizontalMixMatrix,            t); % Horizontal Mixing @ t (m3/d); (3D matrix: 1 X SourceBox X DestinationBox); all positive values

HorizontalMix_OffshoreSurface_t    = interp1(t_grid, HorizontalMix_OffshoreSurface,    t); % mixing rate across oceanic boundary @ t (m3/d); (positive scaler);
HorizontalMix_OffshoreSubSurface_t = interp1(t_grid, HorizontalMix_OffshoreSubSurface, t); % mixing rate across oceanic boundary @ t (m3/d); (positive scaler);

% step 2f: light terms @ t ------------------------------------------------
MLD_t                     = interp1(t_grid, MLD,                            t); % Mixed Layer Depth @ t; (m); (scaler)
Io_t                      = interp1(t_grid, Io,                             t); % surface PAR light intensity @ t; daily integrated solar raditation at ocean surface averaged across 24 hours; (W m^-2 h^-1); (scaler); 

% step 2g: sinking fractions @ t ------------------------------------------
SinkFrac_t                = interp1(t_grid, SinkFrac,                       t); % sinking fraction of source box production @ t; (unitless); (1 X num_E2Egroups X source boxes)

% % step 2h: scenario rules @ t ---------------------------------------------
% Scenario_scaler_t         = interp1(t_grid, Scenario_scaler,                t) - 1; % QQQ remove -1 ; Production_input @ t; external input to model domain; (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X num_boxes)





% % % % % *************************************************************************
% % % % % QQQ turned off 8-11-2016 in order to preserve "labile benthic detritus" group
% % % % % STEP 3: put BioenergeticBudget terms @ t back into EnergyBudget_matrix---
% % % % % step 3a: re-scale predation to account for any defined seasonal changes in non-predation terms
% % % % %          (this section will re-normalize each clm of EnergyBudget_matrix to 1)
% % % % %          NOTE: Current code does not account for the possibility of
% % % % %                defined seasonal changes in flows to NON-terminal detritus pools. Step 3e is used
% % % % %                to account for flow to NON-terminal detritus pools among fishery and detritus clms.
% % % % QBudget_NonPredation_t                            = QBudget_feces_t + QBudget_metabolism_t + QBudget_eggs_t + QBudget_Senescence_t + (QBudget_BA_t + QBudget_EM_t); % NOTE (5/6/15): fishery removals are included in QBudget_EM_t; (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % new_predation                                     = 1 - QBudget_NonPredation_t;            % (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % predation_scaler                                  = new_predation ./ QBudget_predation_t;  % (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % predation_scaler(isnan(predation_scaler))         = 1;                                     % filter out div/0 errors (from fisheries which have 0 predation); (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % predation_scaler(isinf(predation_scaler))         = 1;                                     % filter out div/0 errors (from fisheries which have 0 predation); (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % predation_scaler(predation_scaler < 0)            = 0;                                     % filter out any negative predation values; (3D matrix: 1 X num_E2Egroups X num_boxes)
% % % % predation_scaler_repmat                           = repmat(predation_scaler, [E2E_rows, 1, 1]); % replicate down rows; (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
% % % % predation_scaler_repmat(looky_DETRITUS, :, :)     = 1;                                     % protect the original detritus terms (to save the original non-terminal detritus values); (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
% % % % EnergyBudget_matrix_fishery                       = EnergyBudget_matrix(:, looky_fishery, :);                                   % hold onto the fisheries clms
% % % % EnergyBudget_matrix_termDetr                      = EnergyBudget_matrix(:, [looky_terminalPlgcDetr looky_terminalBnthDetr], :); % hold onto the terminal detritus clms
% % % % EnergyBudget_matrix_Detr                          = EnergyBudget_matrix(:, looky_DETRITUS, :);                                  % hold onto ALL detritus clms
% % % % EnergyBudget_matrix                               = EnergyBudget_matrix .* predation_scaler_repmat;
% % % % 
% % % % % step 3b: BioenergeticBudget @ t metabolism into EnergyBudget_matrix -----
% % % % EnergyBudget_matrix(looky_plgcNH4, :, :)          = QBudget_metabolism_t .* ExcretionFate(1, :, :); % excretion into surface waters; (fraction)
% % % % EnergyBudget_matrix(looky_bnthNH4, :, :)          = QBudget_metabolism_t .* ExcretionFate(2, :, :); % excretion into sub-surface waters & benthos; (fraction)
% % % % 
% % % % % step 3c: BioenergeticBudget @ t terminal detritus (sum of feces and senescence) into EnergyBudget_matrix
% % % % EnergyBudget_matrix(looky_terminalPlgcDetr, :, :) = (QBudget_feces_t .* DetritusFate_feces(1, :, :)) + (QBudget_Senescence_t .* DetritusFate_senescence(1, :, :)); % excretion into surface waters; (fraction)
% % % % EnergyBudget_matrix(looky_terminalBnthDetr, :, :) = (QBudget_feces_t .* DetritusFate_feces(2, :, :)) + (QBudget_Senescence_t .* DetritusFate_senescence(2, :, :)); % excretion into sub-surface waters & benthos; (fraction)
% % % % % QQQ here I need a detritus fate variable that includes non-terminal detritus groups
% % % % 
% % % % % step 3d: BioenergeticBudget @ t egg production into EnergyBudget_matrix -
% % % % EnergyBudget_matrix(looky_eggs, :, :)             = repmat(QBudget_eggs_t, num_eggs, 1) .* EggFate; % egg production (should work for for [] eggs or for multiple eggs); (fraction)
% % % % 
% % % % % step 3e: plug original fishery & terminal detritus clms back into EnergyBudget_matrix
% % % % %          NOTE: these two lines needed for fishery and detritus groups (necessary because these groups include contributions to non-terminal detritus pools)
% % % % EnergyBudget_matrix(:, looky_fishery, :)          = EnergyBudget_matrix_fishery; % plug the original fishery           clms back into EnergyBudget_matrix
% % % % EnergyBudget_matrix(:, looky_DETRITUS, :)         = EnergyBudget_matrix_Detr;    % plug the original terminal detritus clms back into EnergyBudget_matrix
% % % % % EnergyBudget_matrix(:, [looky_terminalPlgcDetr looky_terminalBnthDetr], :) = EnergyBudget_matrix_termDetr; % plug the original terminal detritus clms back into EnergyBudget_matrix

% step 3f: apply box type rules -------------------------------------------
EnergyBudget_matrix                               = EnergyBudget_matrix .* EnergyBudget_BoxType; % adjust sub-surface EnergyBudget matrix; allow only certain recycling processes in sub-surface boxes; (fraction); (3D matrix: consumers X producers X num_boxes)





% *************************************************************************
% STEP 4: prepare Production repmat matrices for array multiplication------
% step 4a: pool pelagic NH4 & benthic NH4 in all boxes --------------------
%          This is done to add any benthic NH4 that was advected & mixed from sub-surface into surface boxes into the pelagic NH4 pool
%          (QQQ this step is redundant with final step in 7d???)
% ProductionRates_t(looky_plgcNH4, 1, :) = ProductionRates_t(looky_plgcNH4, :, :) + ProductionRates_t(looky_bnthNH4, :, :); % (mmole N/m3/d); (3D matrix: num_E2Egroups X 1 X num_boxes)
% ProductionRates_t(looky_bnthNH4, 1, :) = 0; % zero the benthic NH4 pool after adding benthic NH4 to the pelagic NH4 pool; (mmole N/m3/d); (3D matrix: num_E2Egroups X 1 X num_boxes)

% step 4a: apply Scenario_scaler_t to ProductionRates_t -------------------
% % % Scenario_scaler_t           = reshape(Scenario_scaler_t, [E2E_rows 1 num_boxes]); % (3D matrix: num_E2Egroups X 1 X num_boxes) QQQ check that reshaping is correct
% % % % ProductionRates_t           = ProductionRates_t .* Scenario_scaler_t;                  % QQQ deactivated ... ; (mmole N/m3/d); (3D matrix: num_E2Egroups X 1 X num_boxes)
% % % 
% % % % QBudget_EM_t = QBudget_EM_t + (-1*Scenario_scaler_t); % QQQ an attempt to get NO3 biomass doubled in box 5





% step 4b: reshape and repmat ProductionRates_t ---------------------------
ProductionRatesC_t_repmat   = repmat(ProductionRates_t,           [1, E2E_clms, 1]); % NOTE: vertical vectors (num_E2Egroups X num_E2Egroups X num_boxes) replicated across clms; (mmole N/m3/d)
ProductionRates_t_transpose = reshape(ProductionRates_t,  [1, E2E_clms, num_boxes]); % transpose to 3D; (3D matrix of HORIZONTAL vectors: 1 X num_E2Egroups X num_boxes); (mmole N/m3/d)
ProductionRatesP_t_repmat   = repmat(ProductionRates_t_transpose, [E2E_rows, 1, 1]); % NOTE: transpose to horizontal vectors (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes) replicated down rows; (mmole N/m3/d)





% *************************************************************************
% STEP 5: biomasses and nutrient concentrations @ t------------------------
% FFF Can I keep a running biomass time-series?
% step 5a: B @ t = Q / (Q/B) ----------------------------------------------
Biomass_t                   = ProductionRates_t ./ reshape(SpecificConsumptionRate_t, [E2E_rows, 1, num_boxes]); % use Q/B value to convert rates to standing stock biomass; (mmole N/m3); (3D matrix: num_E2Egroups X 1 X num_boxes); NOTE transpose of SpecificConsumptionRate_t to vertical vector
Biomass_PrimaryProducer_t   = Biomass_t(looky_PrimaryProducer, 1, :);                             % primary producer biomasses; (mmole N/m3); (3D matrix: num_PrimaryProducers X 1 X num_boxes)





% *************************************************************************
% STEP 6: calculate advection & mixing exchange rates between boxes and out of domain
% step 6a: identify boxes that are destination boxes for river or ocean input from outside model domain
%          NOTE: can use this to identify times of downwelling or upwelling (upwelling is POSITIVE)
%          NOTE: Can put this all into the physics function and remove from the ODE? (No, may cause trouble when t interval is not an integer in solver)
% calculate advection rates INTO model domain
Advection_DestinationSum                                = sum(Advection_t, 2);                             % advected volume (m3/d); (3D matrix: 1 X 1 X DestinationBoxes)
RiverFlux_DestinationSum                                = sum(RiverFlux_t, 2);                             % advected volume (m3/d); (3D matrix: 1 X 1 X DestinationBoxes)
%   NOTE: all advection terms except those from external inputs (i.e. ocean or river) cancel out; 
% Advection_DestinationSum(1, 1, looky_SubSurfaceBoxes) = 0;                      % QQQ use this line to NOT zero out the deep water ocean destination box (i.e., use this line to allow advection of empty water at depth)
looky_OceanSourceBox                                    = find(Advection_DestinationSum(1, 1, :) < 0);     % box number of box receiving advection from outside
looky_RiverSourceBox                                    = find(RiverFlux_DestinationSum(1, 1, :) < 0);     % box number of box receiving advection from outside

% step 6b: setup box volume & biomass variables ---------------------------
BoxVolume_t                                             = reshape(BoxVolume_t,    [1, 1, num_boxes]); % (m3); (3D matrix: 1 X 1 X num_boxes)
BoxVolume_t                                             = repmat(BoxVolume_t,     [E2E_rows, 1, 1]);  % (m3); (3D matrix: E2E_rows X 1 X num_boxes)
BoxBiomass_A                                            = repmat(Biomass_t(:, :), [1, 1, num_boxes]); % box biomass (mmole N/m3); replicate boxes as layers; (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
BoxBiomass_B                                            = repmat(Biomass_t,       [1, num_boxes, 1]); % box biomass (mmole N/m3); replicate boxes as clms; (3D matrix: num_E2Egroups X DestinationBox X SourceBox)
BoxBiomass_diff                                         = BoxBiomass_A - BoxBiomass_B;                % difference in biomass between boxes: box A - box B; (mmole N/m3); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
BoxBiomass_ocean                                        = BoxBiomass_A;                               % box biomass (mmole N/m3); (3D matrix: num_E2Egroups X SourceBox X DestinationBox); boxes replicated as layers
BoxBiomass_river                                        = BoxBiomass_A;                               % box biomass (mmole N/m3); (3D matrix: num_E2Egroups X SourceBox X DestinationBox); boxes replicated as layers
% do not advect empty water into model domain, zero all nonNO3 groups in Box receiving water from outside domain
% NOTE: this still lets advection of biomass OUT of model domain
% NOTE: for Advection_t & RiverFlux_t -- there is no external biomass pool being imported into the domain, there is simply no import of empty water into the model domain. The assumption is that anything imported INTO an offshore box (or river-adjacent box) is the same as what is already in the box
BoxBiomass_ocean(looky_nonNO3, looky_OceanSourceBox, looky_OceanSourceBox) = 0; % do not advect empty water into model domain, zero all nonNO3 groups in Box receiving water from outside domain
BoxBiomass_river(looky_nonNO3, looky_RiverSourceBox, looky_RiverSourceBox) = 0; % do not advect empty water into model domain, zero all nonNO3 groups in Box receiving water from outside domain

BoxBiomass_OffshoreSurfaceBox                           = BoxBiomass_ocean(:, looky_OffshoreSurfaceBox,    looky_OffshoreSurfaceBox);    % (vertical vector: num_E2Egroups X 1); (mmole N/m3);
BoxBiomass_OffshoreSubSurfaceBox                        = BoxBiomass_ocean(:, looky_OffshoreSubSurfaceBox, looky_OffshoreSubSurfaceBox); % (vertical vector: num_E2Egroups X 1); (mmole N/m3);
BoxBiomass_diff_OffshoreSurfaceBox                      = BoxBiomass_OceanSurface    - BoxBiomass_OffshoreSurfaceBox;                    % (vertical vector: num_E2Egroups X 1); (mmole N/m3);
BoxBiomass_diff_OffshoreSubSurfaceBox                   = BoxBiomass_OceanSubSurface - BoxBiomass_OffshoreSubSurfaceBox;                 % (vertical vector: num_E2Egroups X 1); (mmole N/m3);
BoxBiomass_diff_OffshoreSurfaceBox(looky_nonNO3)        = 0;                                                                             % zero out the non-nutrients in the difference vectors
BoxBiomass_diff_OffshoreSubSurfaceBox(looky_nonNO3)     = 0;                                                                             % zero out the non-nutrients in the difference vectors; (this sub-surface vector should already be all 0)

% step 6c: advection ------------------------------------------------------
AdvectedVolume                                          = repmat(Advection_t, [E2E_clms, 1, 1]);               % volume advected (m3/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
AdvectedBiomass                                         = AdvectedVolume .* BoxBiomass_ocean;                  % (mmole N/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
AdvectedBiomass                                         = sum(AdvectedBiomass, 2);                             % net input to each destination box; (mmole N/d); (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetAdvection_t                                          = AdvectedBiomass ./ BoxVolume_t;                      % normalize by DestinationBox volume; (mmole N/m3/d); (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetAdvection_t                                          = reshape(NetAdvection_t,  [1, E2E_clms, num_boxes]);  % transpose to horizontal vectors; (1 X num_E2Egroups X DestinationBox); (mmole N/m3/d)

% step 6d: river flux -----------------------------------------------------
AdvectedVolume                                          = repmat(RiverFlux_t, [E2E_clms, 1, 1]);               % volume advected (m3/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
AdvectedBiomass                                         = AdvectedVolume .* BoxBiomass_river;                  % (mmole N/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
AdvectedBiomass                                         = sum(AdvectedBiomass, 2);                            
NetRiverFlux_t                                          = AdvectedBiomass ./ BoxVolume_t;                      % sum across SourceBoxes and normalize by DestinationBox volume; (mmole N/m3/d); (num_E2Egroups X 1 X DestinationBox)
NetRiverFlux_t                                          = reshape(NetRiverFlux_t,  [1, E2E_clms, num_boxes]);  % transpose to horizontal vectors; (1 X num_E2Egroups X DestinationBox); (mmole N/m3/d)

% step 6e: add advection and river (alongshore) fluxes --------------------
NetAdvection_t                                          = NetAdvection_t + NetRiverFlux_t;                     % add ocean advection + river flux rates; (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X DestinationBox)

% step 6f: vertical mixing ------------------------------------------------
VerticalMixVolume                                       = repmat(VerticalMix_t, [E2E_clms, 1, 1]);             % volume mixed; (m3/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
VerticalMixBiomass                                      = VerticalMixVolume .* BoxBiomass_diff;                % (mmole N/d); (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
NetVerticalMix_t                                        = sum(VerticalMixBiomass, 2);
NetVerticalMix_t                                        = NetVerticalMix_t ./ BoxVolume_t;                     % sum across SourceBoxes and normalize by DestinationBox volume; (mmole/m3/d); (num_E2Egroups X 1 X DestinationBox)
NetVerticalMix_t                                        = reshape(NetVerticalMix_t, [1, E2E_clms, num_boxes]); % transpose to horizontal vectors; (mmole N/m3/d); (1 X num_E2Egroups X DestinationBox)

% step 6g: horizontal mixing ----------------------------------------------
%          NOTE: extra coding is for horizontal mixing of nutrients across oceanic boundaries
%          NOTE: unlike NCC which uses advection-driven "Production_input", ...
%                horizontal mixing depends upon the domain box biomasses @ t.
HorizontalMixVolume                                     = repmat(HorizontalMix_t,                    [E2E_clms, 1, 1]); % volumetric mixing rate @ t;                         (m3/d);         (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
HorizontalMixVolume_OceanSurface                        = repmat(HorizontalMix_OffshoreSurface_t,    [E2E_clms, 1, 1]); % volumetric mixing rate across oceanic boundary @ t; (m3/d);         (vertical vector: num_E2Egroups X 1)
HorizontalMixVolume_OceanSubSurface                     = repmat(HorizontalMix_OffshoreSubSurface_t, [E2E_clms, 1, 1]); % volumetric mixing rate across oceanic boundary @ t; (m3/d);         (vertical vector: num_E2Egroups X 1)
HorizontalMixBiomass                                    = HorizontalMixVolume                 .* BoxBiomass_diff;                       %                                     (mmole N/d);    (3D matrix: num_E2Egroups X SourceBox X DestinationBox)
HorizontalMixBiomass_OceanSurface                       = HorizontalMixVolume_OceanSurface    .* BoxBiomass_diff_OffshoreSurfaceBox;    %                                     (mmole N/d);    (vertical vector: num_E2Egroups X 1)
HorizontalMixBiomass_OceanSubSurface                    = HorizontalMixVolume_OceanSubSurface .* BoxBiomass_diff_OffshoreSubSurfaceBox; %                                     (mmole N/d);    (vertical vector: num_E2Egroups X 1)
NetHorizontalMix_t                                      = sum(HorizontalMixBiomass, 2);                                                 % sum across SourceBoxes;             (mmole N/d);    (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetHorizontalMix_t(:, 1, looky_OffshoreSurfaceBox)      = NetHorizontalMix_t(:, 1, looky_OffshoreSurfaceBox)    + HorizontalMixBiomass_OceanSurface;       % (mmole N/d);    (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetHorizontalMix_t(:, 1, looky_OffshoreSubSurfaceBox)   = NetHorizontalMix_t(:, 1, looky_OffshoreSubSurfaceBox) + HorizontalMixBiomass_OceanSubSurface;    % (mmole N/d);    (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetHorizontalMix_t                                      = NetHorizontalMix_t ./ BoxVolume_t;                                            % normalize by DestinationBox volume; (mmole/m3/d);   (3D matrix: num_E2Egroups X 1 X DestinationBox)
NetHorizontalMix_t                                      = reshape(NetHorizontalMix_t, [1, E2E_clms, num_boxes]);                        % transpose to horizontal vectors;    (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X DestinationBox)

% step 6h: apply retention scalers ----------------------------------------
%          NOTE: the greater the RetentionScaler, the more resistant to retention; 1 means no advection, 0 means purely planktonic
NetAdvection_t                                          = NetAdvection_t     .* (1 - RetentionScaler); % (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X DestinationBox)
NetVerticalMix_t                                        = NetVerticalMix_t   .* (1 - RetentionScaler); % (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X DestinationBox)
NetHorizontalMix_t                                      = NetHorizontalMix_t .* (1 - RetentionScaler); % (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X DestinationBox)


% % % temp_term = (ProductionRates_t .* Scenario_scaler_t); % QQQ
% % % temp_term = reshape(temp_term, [1 E2E_clms num_boxes]); % QQQ
% % % NetAdvection_t = NetAdvection_t + temp_term; % QQQ


% step 6i: sinking --------------------------------------------------------
SinkFrac_repmat      = repmat(SinkFrac_t, [num_boxes, 1, 1]); % sinking fraction of source box production @ t; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes: replicated down rows)
SinkFrac_surface     = SinkFrac_repmat  .* SinkLink_surface;  % sinking fraction of surface     source box production @ t; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes)
SinkFrac_benthos     = SinkFrac_repmat  .* SinkLink_benthos;  % sinking fraction of sub-surface source box production @ t; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes)
SinkProd_surface     = SinkFrac_surface .* repmat(ProductionRates_t_transpose, [num_boxes, 1, 1]); % production sinking from each surface source box to each destiny box; (mmole N/m3/d); (3D matrix: destination boxes X num_E2Egroups X source boxes)
SinkProd_benthos     = SinkFrac_benthos .* repmat(ProductionRates_t_transpose, [num_boxes, 1, 1]); % production sinking from each surface source box to each destiny box; (mmole N/m3/d); (3D matrix: destination boxes X num_E2Egroups X source boxes)

% normalize production transfer by box source and destination box volumes 
BoxVolume_source      = reshape(BoxVolume_t, [1, E2E_clms, num_boxes]);
BoxVolume_source      = repmat(BoxVolume_source, [num_boxes, 1, 1]);      % (m3); (3D matrix: num_boxes X E2E_clms X num_boxes); source boxes replicated across layers
BoxVolume_destination = repmat(squeeze(BoxVolume_t)', [1, 1, num_boxes]); % (m3); (3D matrix: num_boxes X E2E_clms X num_boxes); destination boxes replicated down rows
SinkProd_surface      = SinkProd_surface .* BoxVolume_source; % absolute production sinking OUT of each SOURCE box; (mmole N/d); (3D matrix: destination boxes X num_E2Egroups X source boxes)
SinkProd_surface      = SinkProd_surface ./ BoxVolume_destination; % sinking production INTO each DESTINATION box; (mmole N/m3/d); (3D matrix: destination boxes X num_E2Egroups X source boxes)

NetSinkProd_surface  = sum(SinkProd_surface, 3); % net gain or loss from each box via sinking; (mmole N/m3/d); (2D matrix: destination boxes X num_E2Egroups)
NetSinkProd_benthos  = sum(SinkProd_benthos, 3); % net gain or loss from each box via sinking; (mmole N/m3/d); (2D matrix: destination boxes X num_E2Egroups)
NetSinking_t         = NetSinkProd_surface;      % net gain or loss from each box via sinking; (mmole N/m3/d); (2D matrix: destination boxes X num_E2Egroups)
% NetSinking_t         = NetSinkProd_surface + NetSinkProd_benthos; % NOTE that we don't want to include sinking from benthos boxes here because this is not a transfer between boxes but rather a transfer within a box from plankto to detritus pools; net gain or loss from each box via sinking; (mmole N/m3/d); (2D matrix: destination boxes X num_E2Egroups)
NetSinking_t         = reshape(NetSinking_t',  [1, E2E_clms, num_boxes]); % transpose to horizontal vectors; (1 X num_E2Egroups X DestinationBox)

% % correct EnergyBudget matrix for sinking losses from model boxes. Choose only ONE of these options
% %   option 1: Consider sinking from any box to have ALREADY been accounted for as senescence mortality. 
% %             Therefore, removal of production as sinking transfer from
% %             surface to sub-surface boxes needs to be removed from the
% %             scenescence terms in the EnergyBudget matrix for surface
% %             boxes. (Sinking from sub-surface boxes is already included in the EnergyBudget as part of senescence mortality)
% SinkFrac_surfaceFLAT = sum(SinkFrac_surface(looky_SurfaceBoxes, :, :), 1); % flatten 3D matrix; net sinking fractions of surface source box production @ t; (unitless); (2D matrix: 1 X num_E2Egroups X source boxes)
% EnergyBudget_matrix([looky_terminalPlgcDetr; looky_terminalBnthDetr], :, :) = EnergyBudget_matrix([looky_terminalPlgcDetr; looky_terminalBnthDetr], :, :) + (repmat(SinkFrac_surfaceFLAT, [2, 1, 1]) .* repmat(DetritusFate_senescence, [1, 1, num_boxes])); % remove fraction transfered from surface to bottom boxes from senescence fraction in EnergyBudget matrix

%   option 2: Consider sinking from any box to have NOT already been accounted for as senescence mortality.
%             Therefore, sinking of production from sub-surface boxes needs
%             to be added to the flow to benthic detritus in the EnergyBudget matrix. 
%             (sinking from surface boxes is already removed as transfer of production
%             between boxes as NetSinking_t)
SinkFrac_benthosFLAT = sum(SinkFrac_benthos, 1); % flatten 3D matrix; net sinking fractions of sub-surface source box production @ t; (unitless); (2D matrix: 1 X num_E2Egroups X source boxes)
SinkFrac_surfaceFLAT = sum(SinkFrac_surface(looky_SurfaceBoxes, :, :), 1); % flatten 3D matrix; net sinking fractions of sub-surface source box production @ t; (unitless); (2D matrix: 1 X num_E2Egroups X source boxes)

% normalize columns to accomodate sink fractions
% sinking losses need to be subtracted from the production distribution to consumers
% (sinking gains do not need to be added to distribution to consumers)
PredFrac                                                         = sum(EnergyBudget_matrix([looky_ANYConsumer; looky_fishery], :, :), 1);
predation_scaler_SubSurface                                      = (PredFrac + SinkFrac_benthosFLAT) ./ PredFrac; % (NOTE: addition here as SinkFrac_benthosFLAT are negative terms)
predation_scaler_SubSurface(predation_scaler_SubSurface < 0)     = 0; % filter out any negative predation values (FFF then re-normalize clms again to trim down non-pred terms and bring back to 100%)
predation_scaler_SubSurface(isnan(predation_scaler_SubSurface))  = 1; % filter out div/0 errors
predation_scaler_SubSurface(isinf(predation_scaler_SubSurface))  = 1; % filter out div/0 errors
predation_scaler_SubSurface                                      = repmat(predation_scaler_SubSurface, [E2E_rows, 1, 1]);
EnergyBudget_matrix([looky_ANYConsumer; looky_fishery], :, :)    = EnergyBudget_matrix([looky_ANYConsumer; looky_fishery], :, :) .* predation_scaler_SubSurface([looky_ANYConsumer; looky_fishery], :, :);
EnergyBudget_matrix(looky_terminalBnthDetr, :, :)                = EnergyBudget_matrix(looky_terminalBnthDetr, :, :) + ((-1)*SinkFrac_benthosFLAT); % add SinkFrac_benthos to flow to benthic detritus in EnergyBudget_matrix (note sign change)

predation_scaler_Surface                                         = (PredFrac + SinkFrac_surfaceFLAT) ./ PredFrac; % (NOTE: addition here as SinkFrac_surfaceFLAT are negative terms)
predation_scaler_Surface(predation_scaler_Surface < 0)           = 0; % filter out any negative predation values (FFF then re-normalize clms again to trim down non-pred terms and bring back to 100%)
predation_scaler_Surface(isnan(predation_scaler_Surface))        = 1; % filter out div/0 errors
predation_scaler_Surface(isinf(predation_scaler_Surface))        = 1; % filter out div/0 errors
predation_scaler_Surface                                         = repmat(predation_scaler_Surface, [E2E_rows, 1, 1]);
EnergyBudget_matrix([looky_ANYConsumer; looky_fishery], :, looky_SurfaceBoxes) = EnergyBudget_matrix([looky_ANYConsumer; looky_fishery], :, looky_SurfaceBoxes) .* predation_scaler_Surface([looky_ANYConsumer; looky_fishery], :, looky_SurfaceBoxes);





% *************************************************************************
% STEP 7: calculate Consumption rate matrix ------------------------------
% step 7a: arena functional responses -------------------------------------
%   form A: Eqtn. 11 in Steele & Ruzicka 2011 -------------------------------
FunctionalResponse = ((1+FunctionalResponseParams) .* ProductionRatesC_t_repmat) ./ (FunctionalResponseParams .* ProductionC_initial_repmat + ProductionRatesC_t_repmat); % (matrix aligned with EnergyBudget_matrix); (unitless); (3D matrix: CONSUMER num_E2Egroups X PRODUCER num_E2Egroups X num_boxes)
FunctionalResponse(isnan(FunctionalResponse)) = 1; % catch NaNs and set to FunctionalResponse = 1 (caused by 0 biomass cells in sub-surface boxes and possibly by dummy variable NaNs)

%   form B: a purely quadratic form -----------------------------------------
        % FunctionalRelation_GROUP    = ProductionGROUP_t    / Production_initial(current_GROUP);
        % FunctionalRelation_PREDATOR = ProductionPREDATOR_t / Production_initial(current_PREDATOR);

% QQQ turn off specific flow pathways
% FunctionalResponse(looky_ANYConsumer, :, :)                                      = 0; % QQQ turn off grazing for de-bugging
% FunctionalResponse([looky_terminalPlgcDetr; looky_terminalBnthDetr], :, :)       = 0; % QQQ turn off scenescence for de-bugging
% FunctionalResponse([looky_plgcNH4; looky_bnthNH4], :, :)                         = 0; % QQQ turn off metabolism flow to nutrients
% FunctionalResponse([looky_eggs], :, :)                                           = 0; % QQQ turn off eggs for de-bugging
% FunctionalResponse([looky_fishery], :, :)                                        = 0; % QQQ turn off fisheries for de-bugging
% FunctionalResponse(looky_ANYConsumer, looky_PrimaryProducer, :)                  = 0; % QQQ turn off grazing on phyto for de-bugging
% FunctionalResponse(looky_ANYConsumer, looky_PrimaryProducer, :)                  = FunctionalResponse(looky_ANYConsumer, looky_PrimaryProducer, :) * 0.8; % QQQ turn down grazing on phyto for de-bugging
% FunctionalResponse([looky_plgcNH4; looky_bnthNH4], [1:101], :)                   = 0; % QQQ turn off metabolism flow to nutrients EXCEPT FOR DETRITUS
% FunctionalResponse([looky_terminalPlgcDetr; looky_terminalBnthDetr], [6:end], :) = 0; % QQQ turn off scenescence (except phyto) for de-bugging

% EnergyBudget_matrix([looky_plgcNH4 looky_bnthNH4], :, :) = 0; % QQQ turn off all NH4



% step 7b: standard calculation using ECOTRAN EnergyBudget_matrix (Acp) ---
%          NOTE: this Q_cp is functional on its own, with or without the
%          Michaelis-Menton option of step 8c
Q_cp                                          = EnergyBudget_matrix .* FunctionalResponse .* ProductionRatesP_t_repmat; % (mmole N/m3/d); (3D matrix: E2E_rows X E2E_clms X num_boxes)

% % step 7c: Michaelis-Menton uptake kinetics for phytoplankton -------------
% %          NOTE: For use when driving primary production with Michaelis-Menten
% %          NOTE: deactivate all step 7c lines when driving primary production using EnergyBudget_matrix
% %          NOTE: PB (& QB) for nutrients = 1, so these rates (ProductionRatesP_t_repmat) are same as concentrations
% %          NOTE: these nutrient concentration variables are used only in Michaelis-Menten functional response calcs
% NO3_t               = ProductionRatesP_t_repmat(looky_PrimaryProducer, looky_NO3,     1:num_boxes); % (mmole N/m3); (3D matrix: num_PrimaryProducers X 1 X num_boxes) repeated down rows;
% plgcNH4_t           = ProductionRatesP_t_repmat(looky_PrimaryProducer, looky_plgcNH4, 1:num_boxes); % (mmole N/m3); (3D matrix: num_PrimaryProducers X 1 X num_boxes) repeated down rows;
% bnthNH4_t           = ProductionRatesP_t_repmat(looky_PrimaryProducer, looky_bnthNH4, 1:num_boxes); % (mmole N/m3); (3D matrix: num_PrimaryProducers X 1 X num_boxes) repeated down rows;
% NH4_t               = plgcNH4_t + bnthNH4_t;                                % (mmole N/m3); (3D matrix: num_PrimaryProducers X 1 X num_boxes) repeated down rows;
% NH4fraction_bnth    = bnthNH4_t ./ NH4_t;                                   % value between 0 - 1; (3D matrix: num_PrimaryProducers X 1 X num_boxes) repeated down rows;
% NH4fraction_bnth(isnan(NH4fraction_bnth)) = 0;                              % change div/0 errors to 0
% 
% [Q_cp_NO3, Q_cp_NH4, Q_cp_plgcNH4, Q_cp_bnthNH4, PB_MichaelisMenten] = ...
%     f_MichaelisMenten_05152016(ODEinput, Biomass_PrimaryProducer_t, NO3_t, NH4_t, NH4fraction_bnth, Io_t, MLD_t);
% 
% Q_cp(looky_PrimaryProducer, looky_NO3, :)     = Q_cp_NO3;                  % paste in Michaelis-Menten         NO3 uptake by primary producers; (mmole N/m3/d); (3D matrix: E2E_rows X E2E_clms X num_boxes)
% Q_cp(looky_PrimaryProducer, looky_plgcNH4, :) = Q_cp_plgcNH4;              % paste in Michaelis-Menten pelagic NH4 uptake by primary producers; (mmole N/m3/d); (3D matrix: E2E_rows X E2E_clms X num_boxes)
% Q_cp(looky_PrimaryProducer, looky_bnthNH4, :) = Q_cp_bnthNH4;              % paste in Michaelis-Menten benthic NH4 uptake by primary producers; (mmole N/m3/d); (3D matrix: E2E_rows X E2E_clms X num_boxes)
% 
% % if macroalgae are present, scale their producion to the total phytoplankton production rate
% %          NOTE: deactivate these step 7c lines when no macroalgae are present in the model
% Q_cp(looky_Macroalgae, looky_NO3, :)          = repmat(sum(Q_cp_NO3), [num_Macroalgae, 1])     .* ProductionFraction_macroalgae; % (3D matrix: E2E_rows X E2E_rows X num_boxes)
% Q_cp(looky_Macroalgae, looky_plgcNH4, :)      = repmat(sum(Q_cp_plgcNH4), [num_Macroalgae, 1]) .* ProductionFraction_macroalgae; % (3D matrix: E2E_rows X E2E_rows X num_boxes)
% Q_cp(looky_Macroalgae, looky_bnthNH4, :)      = repmat(sum(Q_cp_bnthNH4), [num_Macroalgae, 1]) .* ProductionFraction_macroalgae; % (3D matrix: E2E_rows X E2E_rows X num_boxes)





% *************************************************************************
% STEP 8: calculate dy = change in production rate for each group P (t/km2/y2)
% QQQ set various individual terms for debugging
% TransferEfficiency(:, looky_terminalBnthDetr) = 0.1; % default TE for benthic detritus
% QBudget_EM_t = QBudget_EM_t .* 0; % QQQ EM off for debugging - this mostly applies to fisheries; I need to test the affect of including this so that I do not inadvertantly cause fisheries to decline over time due to "exporting" their "biomass"
QBudget_EM_t(1, looky_fishery, :) = QBudget_EM_t(1, looky_fishery, :) .* 0; % set fishery EM to 0 - so that I do not cause fisheries to decline over time due to "exporting" their "biomass" (the export IS their biomass, and discards are accounted for as "feces" in EnergyBudget)


% step 8a: calculate Consumption_INflow & Predation_OUTflow for each group
% NOTE: Predation_OUTflow = ProductionRates_t' if each clm of EnergyBudget_matrix sums to 1 (when FunctionalResponse = 1)
% NOTE: Predation_OUTflow includes metabolism, senescence, & feces
Consumption_INflow                      = sum(Q_cp, 2);                                           % total consumption flow into each consumer C or metabolic demand (mmole N/m3/d); (vertical vectors: num_E2Egroups X 1 X num_boxes)
Consumption_INflow                      = reshape(Consumption_INflow,  [1, E2E_clms, num_boxes]); % transpose to horizontal vectors (1 X num_E2Egroups X num_boxes); (mmole N/m3/d)
Predation_OUTflow                       = sum(Q_cp, 1);                                           % total predation flow out from each producer P (mmole N/m3/d); (horizontal vector) (1 X num_E2Egroups X num_boxes)

% step 8b: move production of benthic NH4 from surface boxes (II, IV) to sub-surface boxes (III, V)
NH4_INflow                              = Consumption_INflow(:, looky_bnthNH4, :);                                % (mmole N/m3/d); (3d matrix: 1 X num_bnthNH4 X source box)
NH4_BoxVolume                           = reshape(BoxVolume_t(looky_bnthNH4, 1, :), [1, num_bnthNH4, num_boxes]); % volume of source box; (m3); (3D matrix: 1 X num_bnthNH4 X source box)
NH4_INflow                              = NH4_INflow .* NH4_BoxVolume;                                            % absolute amount of NH4 in source box; (mmole N/d); (3d matrix: 1 X num_bnthNH4 X source box)
NH4_INflow_repmat                       = repmat(NH4_INflow, [num_boxes, 1, 1]);                                  % (mmole N/d); (3D matrix: destination box X num_bnthNH4 X source box)
NH4_INflow_repmat                       = NH4_INflow_repmat .* SinkLink_surface(:, looky_bnthNH4, :);             % (mmole N/d); (3d matrix: destination box X num_bnthNH4 X source box)
NH4_INflow_repmat                       = sum(NH4_INflow_repmat, 3);                                              % net transfer to destination boxes; (mmole N/d); (2D matrix: destination box X num_bnthNH4)
NH4_BoxVolume                           = squeeze(BoxVolume_t([looky_bnthNH4], 1, :));                            % volume of destination box; (m3); (2D matrix: destination X num_bnthNH4); (if error because of multiple benthic NH4 pools, add transpose ' to BoxVolume_t([looky_bnthNH4], 1, :)')
NH4_transfer                            = NH4_INflow_repmat ./ NH4_BoxVolume;                                     % normalize by destination box volume; NH4 transfer from surface to sub-surface boxes; (mmole N/m3/d); (2D matrix: destination X num_bnthNH4)
NH4_transfer                            = reshape(NH4_transfer', [1, num_bnthNH4, num_boxes]);                    % (mmole N/m3/d); (3d matrix: 1 X num_bnthNH4 X source box)
Consumption_INflow(1, looky_bnthNH4, :) = Consumption_INflow(1, looky_bnthNH4, :) + NH4_transfer;                 % add NH4_transfer to Consumption_INflow to transfer benthic NH4 from surface to sub-surface boxes;  (mmole N/m3/d); (3d matrix: 1 X num_bnthNH4 X source box)

% step 8c: pool pelagic NH4 & benthic NH4 in all boxes --------------------
Consumption_INflow(1, looky_plgcNH4, :) = Consumption_INflow(1, looky_plgcNH4, :) + Consumption_INflow(1, looky_bnthNH4, :); % (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X num_boxes)
Consumption_INflow(1, looky_bnthNH4, :) = 0; % zero out benthic NH4 pool after combining with the pelagic NH4 pool; (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X num_boxes)





% *************************************************************************
% STEP 9: calculate final dy-----------------------------------------------                 
% step 9a: ----------------------------------------------------------------
%           NOTE: dy units (mmole N/m3/d2); (3D matrix: 1 X num_E2Egroups X num_boxes)
%           NOTE: use SpecificConsumptionRate_t
%           NOTE: physical loss taerms here correspond to John's original format
%                     where I use the term: NetAdvection_t + NetVerticalMix_t + NetHorizontalMix_t + NetSinking_t
%                     John used the term: PhysicalLossFraction * currentProductionRate
%                     These are the same in all essentials except that the
%                     new code also allows for physical GAIN as well as LOSS

dy                 = SpecificConsumptionRate_t .* ...
                     ((TransferEfficiency .* (Consumption_INflow + ProductionInput_t)) - ...
                     Predation_OUTflow - ...
                     ((QBudget_EM_t - QBudget_BA_t) .* ProductionRates_t_transpose) + ...
                     NetAdvection_t + NetVerticalMix_t + NetHorizontalMix_t + NetSinking_t);

% dy_evaluate = sum(sum(squeeze(dy))); % QQQ for debugging


% step 9b: stack dy into 1 column vector ----------------------------------
% NOTE: dy MUST be returned as a column vector, because that's the way MATLAB wants it;
dy      = dy(:);                    % (mmole N/m3/d2), (vertical vector: (num_E2Egroups * num_boxes) X 1)
% dy      = round2(dy, dy_precision); % QQQ clean out any tiny trace rounding error




% end ODE function*********************************************************