function FunctionalResponseParams = f_FunctionalResponse_MonteCarlo_09122016(ECOTRAN, ODEinput, CV_FunctionalResponse)
% prepare vulnerability array
% including random functional response parameters
% NOTE: producer "vulnerabilities", m_p, are a characteristic of the CONSUMER
% returns:
%       FunctionalResponseParams        (producer "vulnerabilities", m_p)   (3D matrix: CONSUMERS X prey group X num_boxes) replicated across clms
% revision date: 9-12-2016


% *************************************************************************
% STEP 1: set distribution for random variables----------------------------
distribution_type = 'normal';   % 'uniform'
max_m_p           = 100;          % maximum vulnerability parameter (m_p); arbitrary (JR 7-13-2016)





% *************************************************************************
% STEP 2: unpack variables-------------------------------------------------
FunctionalResponseParams    = ECOTRAN.FunctionalResponseParams; % vulnerability parameter (m_p); (horizontal vector: 1 X EwE groups)
looky_NUTRIENTS             = ODEinput.looky_NUTRIENTS;
looky_DETRITUS              = ODEinput.looky_DETRITUS;
num_E2Egroups               = ODEinput.E2E_clms;
looky_SubSurfaceBoxes       = ODEinput.looky_SubSurfaceBoxes;
num_boxes                   = ODEinput.num_boxes;
num_Nutrients               = length(looky_NUTRIENTS);





% *************************************************************************
% STEP 2: define functional predator-prey relations------------------------
% step 2a: select a "linear" or "ECOSIM" response -------------------------
%          NOTE: to change one half-sat constant for individual groups, change m_p([Grp_rowclm])
%                m_p = 0 is "linear"
%                m_p = 1 is "non-linear" ECOSIM default
m_p                         = [zeros(1, num_Nutrients) FunctionalResponseParams(1, :)]; % append nutrients (to start); choose only the first of the four potential FunctionalResponseParams; (horizontal vector: 1 X num_E2Egroups)





% *************************************************************************
% STEP 3: generate random functional response vector-----------------------
if strcmp(distribution_type, 'uniform')
    
    % option 1: random UNIFORM distribution -------------------------------
    % step 3a: interval over which to draw random vulnerabilities ---------
    interval_low_m_p                                    = m_p - (m_p .* CV_FunctionalResponse);                                         % (horizontal vector: 1 X num_E2Egroups)
    looky_small                                         = find(interval_low_m_p < 0);                                                   % (horizontal vector: 1 X num_E2Egroups)
    interval_low_m_p(looky_small) = 0;
    interval_high_m_p                                   = m_p + (m_p .* CV_FunctionalResponse);                                         % (horizontal vector: 1 X num_E2Egroups)
    looky_big                                           = find(interval_high_m_p > max_m_p); 
    interval_high_m_p(looky_big)                        = max_m_p;                                                                      % (horizontal vector: 1 X num_E2Egroups)    
    
    % step 3b: random vulnerabilities-- -----------------------------------
    random_vector                                       = rand(1, num_E2Egroups);                                                       % draw from UNIFORM distribution; (horizontal vector: 1 X num_E2Egroups)
    MonteCarloVector                                    = interval_low_m_p + ((interval_high_m_p - interval_low_m_p) .* random_vector); % produce the new random m_p vector; (horizontal vector: 1 X num_E2Egroups)
    MonteCarloVector([looky_NUTRIENTS; looky_DETRITUS])	= 0;                                                                            % set so that flow from nutrients & detritus is linear (should already be done, but just in case); (horizontal vector: 1 X num_E2Egroups)

elseif strcmp(distribution_type, 'normal')
    
    % option 2: random NORMAL distribution --------------------------------
    % step 3a: interval over which to draw random vulnerabilities ---------
    interval_low_m_p                                    = zeros(1, num_E2Egroups);         % (horizontal vector: 1 X num_E2Egroups)
    interval_high_m_p                                   = (m_p .* CV_FunctionalResponse);  % (horizontal vector: 1 X num_E2Egroups); NOTE: this is the STDev when using NORMAL distribution
    
    % step 3b: random vulnerabilities -------------------------------------
    random_vector                                       = randn(1, num_E2Egroups);                                                      % draw from NORMAL distribution; (horizontal vector: 1 X num_E2Egroups)
    MonteCarloVector                                    = m_p + (interval_high_m_p .* random_vector);                                   % produce the new random m_p vector; (horizontal vector: 1 X num_E2Egroups)
    MonteCarloVector([looky_NUTRIENTS; looky_DETRITUS]) = 0;                                                                            % set so that flow from nutrients & detritus is linear (should already be done, but just in case); (horizontal vector: 1 X num_E2Egroups)
    looky_small                                         = find(MonteCarloVector < 0);
    MonteCarloVector(looky_small)                       = 0;                                                                            % set negative values to 0
    looky_big                                           = find(MonteCarloVector > max_m_p); 
    MonteCarloVector(looky_big)                         = 1;                                                                            % set too large values to 1
    MonteCarloVector                                    = MonteCarloVector';                                                            % transpose, vulnerabilities are functions of the CONSUMER; (vertical vector: num_E2Egroups X 1)
end





% *************************************************************************
% STEP 4: finalize vulnerabilities-----------------------------------------
m_p_repmat                                       = repmat(MonteCarloVector, [1, num_E2Egroups]); % replicate m_p across clms; (2D matrix: CONSUMERS X prey group)
m_p_repmat(:, [looky_NUTRIENTS; looky_DETRITUS]) = 0;                                            % set so that flow from nutrients & detritus pools is linear; (2D matrix: CONSUMERS X prey group)
m_p_repmat                                       = repmat(m_p_repmat, [1, 1, num_boxes]);        % replicate again for each spatial box; (3D matrix: CONSUMERS X prey group X num_boxes)
m_p_repmat(:, :, looky_SubSurfaceBoxes)          = 0;                                            % functional response terms in sub-surface boxes are all linear; (3D matrix: CONSUMERS X prey group X num_boxes)
FunctionalResponseParams                         = m_p_repmat;                                   % pack to send to ODE; (3D matrix: CONSUMERS X prey group X num_boxes)


% end m-file***************************************************************