function Production_initial = f_InitialProductionRates_05112016(ODEinput, DriverProductionVector, t_current)
% calculate initial or mean production conditions
% takes: 
%       ODEinput
%           EnergyBudget_matrix             consumption fates           (fraction)        (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
%           PhysicalLossFraction            PhysicalLossFraction @ t                      (2D matrix: time X num_E2Egroups)
%           TransferEfficiency                                                            (horizontal vector: 1 X num_E2Egroups)
%           looky_NUTRIENTS
%           looky_plgcNH4
%           looky_bnthNH4
%           BoxVolume                                                   (m3)              (2D matrix: time X num_boxes) (FFF could get this from ECOTRANphysics?)
%           SinkLink_surface                sinking connections from surface to sub-surface boxes	(unitless)	(3D matrix: destination boxes X num_E2Egroups X source boxes); don't confuse with ECOTRANphysics.SinkLink_surface
%           num_boxes                                                                     (scaler)
%       DriverProductionVector              production driver vector for each Box	(mmole N/m3/d)  (3D matrix: 1 X num_E2Egroups X num_boxes)
%       t_current                           day of year
% returns:
%       Production_initial                  initial or mean production rates (mmole N/m3/d)     (2D matrix: num_E2Egroups X num_boxes)
% calls:
%       f_WebProductivityWLoss
% revision date: 5-15-2016


% *************************************************************************
% STEP 1: unpack givens----------------------------------------------------
EnergyBudget_matrix         = ODEinput.EnergyBudget_matrix;
PhysicalLossFraction        = ODEinput.PhysicalLossFraction;    % PhysicalLossFraction at t; (2D matrix: time X groups)
TransferEfficiency          = ODEinput.TransferEfficiency;      % (horizontal vector: 1 X num_E2Egroups)
looky_NUTRIENTS             = ODEinput.looky_NUTRIENTS;
looky_plgcNH4               = ODEinput.looky_plgcNH4;
looky_bnthNH4               = ODEinput.looky_bnthNH4;
% num_NH4                     = ODEinput.num_NH4; % QQQ not sure if I want just bnth NH4 pools or all NH4 pools???
BoxVolume                   = ODEinput.BoxVolume;               % (m3); (2D matrix: time X num_boxes)
SinkLink_surface            = ODEinput.SinkLink_surface; % sinking connections from surface to sub-surface boxes; (unitless); (3D matrix: destination boxes X num_E2Egroups X source boxes)
num_boxes                   = ODEinput.num_boxes;





% *************************************************************************
% STEP 2: prep-------------------------------------------------------------
% step 2a: ----------------------------------------------------------------
[num_E2Egroups, E2E_clms, num_boxes]    = size(EnergyBudget_matrix);
PhysicalLossFraction_t                  = PhysicalLossFraction(t_current, :);   % (horizontal vector: 1 X num_E2Egroups)
Production_initial                      = zeros(num_E2Egroups, num_boxes);      % initialize; (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes)

% step 2b: evaluate type of driver (nutrient vs living group) -------------
% NOTE: TURN OFF NUTRIENT RECYCLING IF DRIVEN BY ANYTHING OTHER THAN NO3
TestInput_base                          = DriverProductionVector; % production driver vector for each Box; (mmole N/m3/d); (3D matrix: 1 X num_E2Egroups X num_boxes)
TestInput_base(:, looky_NUTRIENTS, :)   = [];
TestInput_base                          = sum(TestInput_base, 3); % flatten TestInput_base; if evaluation conditions hold for 1 box, we say that they hold for all boxes
if max(TestInput_base > 0)
    % NOTE: this method cannot handle mixed nutrient & non-nutrient drivers in the same run
%     disp('WARNING: non-NO3 driver. Be sure to deactivate nutrient recycling; set Nutrient columns in production matrix to 0');
    EnergyBudget_matrix(:, looky_NUTRIENTS, :) = 0; % shut off recycling; turn off flow of nutrients to any other box (i.e., primary producers)
% else
%     disp('WARNING: NO3 driver. Be sure to keep nutrient recycling ACTIVATED'); 
end





% *************************************************************************
% step 3: calculate initial production rates for each model box domain ----
% (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes)
%          NOTE: no ZEROS are allowed else NaN errors else NaN errors from functional response equations
%          NOTE: when using the full BioenergeticBudget matrix, 
%                this is really total consumption (Q) flowing into each box rather than production (P)
for box_loop = 1:num_boxes
    current_TransferEfficiency      = TransferEfficiency(1, :, box_loop);
    current_DriverProductionVector  = DriverProductionVector(1, :, box_loop);   % driver poduction vector for current Box; (mmoles N/m3/d); (horizontal vector: 1 X num_groups)
    current_EnergyBudget_matrix     = EnergyBudget_matrix(:, :, box_loop);      % (2D matrix: num_E2Egroups X num_E2Egroups)
    Production_initial(:, box_loop) = f_WebProductivityWLoss(current_TransferEfficiency, current_EnergyBudget_matrix, current_DriverProductionVector, PhysicalLossFraction_t); % production (actually CONSUMPTION) rates as "initial" or "mean" conditions; (mmole N/m3/d); (vertical vector: num_E2Egroups X 1)
end





% *************************************************************************
% step 4: transfer bnthNH4 from surface to appropriate sub-surface box ----
% step 4a: ----------------------------------------------------------------
BoxVolume_t        = BoxVolume(t_current, :);                                       % (m3); (horizontal vector: 1 X num_boxes)
num_bnthNH4        = length(looky_bnthNH4);                                         % FFF redefine here (correct it in the main dynmaic code too)
BoxVolume_repmat   = repmat(BoxVolume_t, [num_bnthNH4, 1]);                         % (m3); (2D matrix: num_bnthNH4 X num_boxes)

NH4_initial        = Production_initial([looky_bnthNH4], :);                        % (mmoles N/m3/d); (2D matrix: num_bnthNH4 X num_boxes)
NH4_initial        = NH4_initial .* BoxVolume_repmat;                               % absolute amount of NH4 in source box; (mmole N/d); (2D matrix: num_bnthNH4 X num_boxes)
NH4_initial        = reshape(NH4_initial, [1, num_bnthNH4, num_boxes]);             % (mmole N/d); (3D matrix: 1 X num_bnthNH4 X num_boxes)

NH4_initial_repmat = repmat(NH4_initial, [num_boxes, 1, 1]);                        % (mmole N/d); (3D matrix: destiny box X num_bnthNH4 X source box)
NH4_initial_repmat = NH4_initial_repmat .* SinkLink_surface(:, [looky_bnthNH4], :); % (mmole N/d); (3D matrix: destination box X num_bnthNH4 X source box)
NH4_initial_repmat = sum(NH4_initial_repmat, 3);                                    % net transfer to destination boxes; (mmole N/d); (2D matrix: destination box X num_bnthNH4)
NH4_initial_repmat = NH4_initial_repmat ./ BoxVolume_repmat';                       % normalize by destination box volume; NH4 transfer from surface to sub-surface boxes; (mmole N/m3/d); (2D matrix: destination X num_bnthNH4)
NH4_initial_repmat = NH4_initial_repmat';                                           % (mmole N/m3/d); (2D matrix: num_bnthNH4 X num_boxes)

% step 4b: add back into Production_initial -------------------------------
Production_initial([looky_bnthNH4], :)      = Production_initial([looky_bnthNH4], :) + NH4_initial_repmat; % initial or mean production rates (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes)
Production_initial(Production_initial < 0)  = 0;                                    % only here for very very small rounding error

% step 4c: pool pelagic NH4 benthic NH4 in all boxes ----------------------
Production_initial(looky_plgcNH4, :)        = Production_initial(looky_plgcNH4, :) + Production_initial(looky_bnthNH4, :); % (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes)
Production_initial(looky_bnthNH4, :)        = 0;                                    % zero out benthic NH4 pool after combining with the pelagic NH4 pool; (mmole N/m3/d); (2D matrix: num_E2Egroups X num_boxes)


% m-file*******************************************************************