function StaticProduction_timeseries = f_StaticProductionTimeseries_09042017(ODEinput, ECOTRANphysics, Production_input, target_box)
% calculate initial or mean production conditions
% takes: 
%       ODEinput
%           EnergyBudget_matrix         consumption fates                       (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
%           PhysicalLossFraction        PhysicalLossFraction                    (2D matrix: time X num_E2Egroups)
%           TransferEfficiency                                                  (horizontal vector: 1 X num_E2Egroups)
%       ECOTRANphysics
%           NO3_doy                     (daily NO3 + NO2 concentration interpolated from monthly means; (mmole N/m3); (2D matrix: num_t X box; I, II, III, IV, V)
%       Production_input                                                        (3D matrix: time X num_E2Egroups X num_boxes)
% returns:
%       StaticProduction_timeseries     production rates @ t (mmole N/m3/d)     (2D matrix: num_E2Egroups X time)
% calls:
%       f_WebProductivityWLoss
% revision date: 9-4-2017


% *************************************************************************
% STEP 1: unpack givens----------------------------------------------------
EnergyBudget_matrix                 = ODEinput.EnergyBudget_matrix;                 % (3D matrix: num_E2Egroups X num_E2Egroups X num_boxes)
TransferEfficiency                  = ODEinput.TransferEfficiency;                  % (3D matrix: 1 X num_E2Egroups X num_boxes)
PhysicalLossFraction                = ODEinput.PhysicalLossFraction;                % PhysicalLossFraction at t; (2D matrix: time X groups)
HorizontalMix_OffshoreSurface       = ODEinput.HorizontalMix_OffshoreSurface;       % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)
HorizontalMix_OffshoreSubSurface    = ODEinput.HorizontalMix_OffshoreSubSurface;    % mixing rate across oceanic boundary = volume mixed per day; (m3/d); (vertical vector: time X 1)
NO3_timeseries                      = ECOTRANphysics.NO3_doy;                       % daily NO3 + NO2 interpolated from monthly means; (mmole N/m3); (2D matrix: num_t X box; I, II, III, IV, V)
looky_OffshoreSurfaceBox            = ODEinput.looky_OffshoreSurfaceBox;
looky_OffshoreSubSurfaceBox         = ODEinput.looky_OffshoreSubSurfaceBox;
looky_NO3                           = ODEinput.looky_NO3;
BoxVolume                           = ODEinput.BoxVolume;                           % (m3); (2D matrix: time X num_boxes)


% *************************************************************************
% STEP 2: prep-------------------------------------------------------------
% step 2a: sizes ----------------------------------------------------------
[num_E2Egroups, toss, num_boxes]    = size(EnergyBudget_matrix);
[num_t,         toss, toss]         = size(Production_input);

% step 2b: select target box ----------------------------------------------
% target_EnergyBudget_matrix          = EnergyBudget_matrix(:, :, target_box);      % (2D matrix: num_E2Egroups X num_E2Egroups)
target_EnergyBudget_matrix          = EnergyBudget_matrix(:, :, 2);      % use box 2 as the default food web in all cases to avoid use of empty food webs (in basin and SubSurface cases); (2D matrix: num_E2Egroups X num_E2Egroups)
target_TransferEfficiency           = TransferEfficiency(1, :, target_box);	      % (horizontal vector: 1 X num_E2Egroups)

% step 2c: initialize final result ----------------------------------------
StaticProduction_timeseries         = zeros(num_E2Egroups, num_t); % (mmole N/m3/d); (2D matrix: num_E2Egroups X num_t)





% *************************************************************************
% STEP 3: rate of NO3 input via horizontal mixing (for bank setting)-------
NO3_timeseries_OceanSurface         = HorizontalMix_OffshoreSurface     .* NO3_timeseries(looky_OffshoreSurfaceBox);    % NO3 driver via horizontal mixing across surface domain boundary with ocean;     (mmoles N/d); (vertical vector: time X 1)
NO3_timeseries_OceanSubSurface      = HorizontalMix_OffshoreSubSurface  .* NO3_timeseries(looky_OffshoreSubSurfaceBox); % NO3 driver via horizontal mixing across sub-surface domain boundary with ocean; (mmoles N/d); (vertical vector: time X 1)
NO3_timeseries_OceanSurface         = NO3_timeseries_OceanSurface    ./ BoxVolume(:, looky_OffshoreSurfaceBox);         % (mmoles N/m3/d); (vertical vector: time X 1); NOTE: assuming ocean volue is same as the boundary box volume
NO3_timeseries_OceanSubSurface      = NO3_timeseries_OceanSubSurface ./ BoxVolume(:, looky_OffshoreSubSurfaceBox);      % (mmoles N/m3/d); (vertical vector: time X 1); NOTE: assuming ocean volue is same as the boundary box volume





% *************************************************************************
% STEP 4: add advection and horizontal mixing NO3 drivers------------------
Production_input(:, looky_NO3, looky_OffshoreSurfaceBox)    = Production_input(:, looky_NO3, looky_OffshoreSurfaceBox)    + NO3_timeseries_OceanSurface;    % (mmoles N/m3/d); (3D matrix: time X num_E2Egroups X num_boxes) 
Production_input(:, looky_NO3, looky_OffshoreSubSurfaceBox) = Production_input(:, looky_NO3, looky_OffshoreSubSurfaceBox) + NO3_timeseries_OceanSubSurface; % (mmoles N/m3/d); (3D matrix: time X num_E2Egroups X num_boxes) 
target_Production_input                                     = Production_input(:, :, target_box);                                                           % select target box; (mmole N/m3/d); (2D matrix: time X num_E2Egroups)


% *************************************************************************
% STEP 5: calculate production rates from static model at each time step---
for time_loop = 1:num_t
    Production_input_t                        = target_Production_input(time_loop, :); % (horizontal vector: 1 X num_E2Egroups)
    PhysicalLossFraction_t                    = PhysicalLossFraction(time_loop, :); % (horizontal vector: 1 X num_E2Egroups)
    StaticProduction_timeseries(:, time_loop) = f_WebProductivityWLoss(target_TransferEfficiency, target_EnergyBudget_matrix, Production_input_t, PhysicalLossFraction_t); % production (actually CONSUMPTION) rates at t; (mmole N/m3/d); (vertical vector: num_E2Egroups X 1)
end


% end m-file***************************************************************